@@ -15,8 +15,6 @@
 #include "OutputEnums.h"
 #include "OutputParams.h"
 
-#include "insets/InsetCode.h"
-
 #include "support/strfwd.h"
 #include "support/types.h"
 
@@ -72,6 +70,10 @@ class FileName;
 class FileNameList;
 }
 
+namespace graphics {
+class PreviewLoader;
+}
+
 
 class Buffer;
 typedef std::list<Buffer *> ListOfBuffers;
@@ -80,7 +82,7 @@ typedef std::set<Buffer *> CloneList;
 
 
 /** The buffer object.
- * This is the buffer object. It contains all the informations about
+ * This is the buffer object. It contains all the information about
  * a document loaded into LyX.
  * The buffer object owns the Text (wrapped in an InsetText), which
  * contains the individual paragraphs of the document.
@@ -90,6 +92,9 @@ typedef std::set<Buffer *> CloneList;
  * minimal, probably not.
  * \author Lars Gullik Bjønnes
  */
+
+class MarkAsExporting;
+
 class Buffer {
 public:
 	/// What type of log will \c getLogName() return?
@@ -107,7 +112,7 @@ public:
 		ReadWrongVersion,
 		ReadFileNotFound,
 		ReadVCError,
-		ReadAutosaveFailure,		
+		ReadAutosaveFailure,
 		ReadEmergencyFailure,
 		ReadNoLyXFormat,
 		ReadDocumentFailure,
@@ -120,12 +125,24 @@ public:
 		ReadOriginal
 	};
 
+	enum ExportStatus {
+		// export
+		ExportSuccess,
+		ExportCancel,
+		ExportError,
+		ExportNoPathToFormat,
+		ExportTexPathHasSpaces,
+		ExportConverterError,
+		// preview
+		PreviewSuccess,
+		PreviewError
+	};
 
 	/// Method to check if a file is externally modified, used by
 	/// isExternallyModified()
 	/**
 	 * timestamp is fast but inaccurate. For example, the granularity
-	 * of timestamp on a FAT filesystem is 2 second. Also, various operations
+	 * of timestamp on a FAT filesystem is 2 seconds. Also, various operations
 	 * may touch the timestamp of a file even when its content is unchanged.
 	 *
 	 * checksum is accurate but slow, which can be a problem when it is
@@ -206,6 +223,10 @@ public:
 	/// emergency or autosave files, one should use \c loadLyXFile.
 	/// /sa loadLyXFile
 	ReadStatus loadThisLyXFile(support::FileName const & fn);
+	/// import a new document from a string
+	bool importString(std::string const &, docstring const &, ErrorList &);
+	/// import a new file
+	bool importFile(std::string const &, support::FileName const &, ErrorList &);
 	/// read a new document from a string
 	bool readString(std::string const &);
 	/// Reloads the LyX file
@@ -221,7 +242,7 @@ private:
 	///
 	void cloneWithChildren(BufferMap &, CloneList *) const;
 	/// save timestamp and checksum of the given file.
-	void saveCheckSum() const;	
+	void saveCheckSum() const;
 	/// read a new file
 	ReadStatus readFile(support::FileName const & fn);
 	/// Reads a file without header.
@@ -229,18 +250,17 @@ private:
 	/// \return \c true if file is not completely read.
 	bool readDocument(Lexer &);
 	/// Try to extract the file from a version control container
-	/// before reading if the file cannot be found. This is only
-	/// implemented for RCS.
+	/// before reading if the file cannot be found.
 	/// \sa LyXVC::file_not_found_hook
 	ReadStatus extractFromVC();
-	/// Reads the first tag of a LyX File and 
+	/// Reads the first tag of a LyX File and
 	/// returns the file format number.
 	ReadStatus parseLyXFormat(Lexer & lex, support::FileName const & fn,
 		int & file_format) const;
 	/// Convert the LyX file to the LYX_FORMAT using
 	/// the lyx2lyx script and returns the filename
 	/// of the temporary file to be read
-	ReadStatus convertLyXFormat(support::FileName const & fn, 
+	ReadStatus convertLyXFormat(support::FileName const & fn,
 		support::FileName & tmpfile, int from_format);
 	//@}
 
@@ -248,7 +268,7 @@ public:
 	/// \name Functions involved in autosave and emergency files.
 	//@{
 	/// Save an autosave file to #filename.lyx#
-	bool autoSave() const;	
+	bool autoSave() const;
 	/// save emergency file
 	/// \return a status message towards the user.
 	docstring emergencyWrite();
@@ -257,11 +277,11 @@ public:
 //private:
 	///
 	void removeAutosaveFile() const;
-	
+
 private:
 	/// Try to load an autosave file associated to \c fn.
 	ReadStatus loadAutosave();
-	/// Try to load an emergency file associated to \c fn. 
+	/// Try to load an emergency file associated to \c fn.
 	ReadStatus loadEmergency();
 	/// Get the filename of the emergency file associated with the Buffer
 	support::FileName getEmergencyFileName() const;
@@ -324,8 +344,7 @@ public:
 			     OutputWhat output = FullSource) const;
 	///
 	void makeLyXHTMLFile(support::FileName const & filename,
-			     OutputParams const & runparams_in,
-			     bool only_body = false) const;
+			     OutputParams const & runparams_in) const;
 	///
 	void writeLyXHTMLSource(odocstream & os,
 			     OutputParams const & runparams_in,
@@ -375,7 +394,7 @@ public:
 	/// Returns the buffer's filename. It is always an absolute path.
 	std::string absFileName() const;
 
-	/// Returns the the path where the buffer lives.
+	/// Returns the path where the buffer lives.
 	/// It is always an absolute path.
 	std::string filePath() const;
 
@@ -398,15 +417,15 @@ public:
 
 	/// \return true if \p child is a child of this \c Buffer.
 	bool isChild(Buffer * child) const;
-	
+
 	/// \return true if this \c Buffer has children
 	bool hasChildren() const;
-	
+
 	/// \return a list of the direct children of this Buffer.
 	/// this list has no duplicates and is in the order in which
 	/// the children appear.
 	ListOfBuffers getChildren() const;
-	
+
 	/// \return a list of all descendents of this Buffer (children,
 	/// grandchildren, etc). this list has no duplicates and is in
 	/// the order in which the children appear.
@@ -438,7 +457,7 @@ public:
 	void validate(LaTeXFeatures &) const;
 
 	/// Reference information is cached in the Buffer, so we do not
-	/// have to check or read things over and over. 
+	/// have to check or read things over and over.
 	///
 	/// There are two caches.
 	///
@@ -446,11 +465,11 @@ public:
 	/// being gathered. This cache is PER BUFFER, and the cache for the
 	/// master essentially includes the cache for its children. This gets
 	/// invalidated when an InsetBibtex is created, deleted, or modified.
-	/// 
+	///
 	/// The other is a cache of the reference information itself. This
 	/// exists only in the master buffer, and when it needs to be updated,
 	/// the children add their information to the master's cache.
-	
+
 	/// Calling this method invalidates the cache and so requires a
 	/// re-read.
 	void invalidateBibinfoCache() const;
@@ -471,6 +490,8 @@ public:
 	/// add a single piece of bibliography info to our cache
 	void addBibTeXInfo(docstring const & key, BibTeXInfo const & bi) const;
 	///
+	void makeCitationLabels() const;
+	///
 	bool citeLabelsValid() const;
 	///
 	void getLabelList(std::vector<docstring> &) const;
@@ -489,6 +510,8 @@ public:
 	///
 	BufferParams & params();
 	BufferParams const & params() const;
+	///
+	BufferParams const & masterParams() const;
 
 	/** The list of paragraphs.
 	    This is a linked list of paragraph, this list holds the
@@ -528,6 +551,8 @@ public:
 	/// Set by buffer_funcs' newFile.
 	void setFullyLoaded(bool);
 
+	/// FIXME: Needed by RenderPreview.
+	graphics::PreviewLoader * loader() const;
 	/// Update the LaTeX preview snippets associated with this buffer
 	void updatePreviews() const;
 	/// Remove any previewed LaTeX snippets associated with this buffer
@@ -565,21 +590,20 @@ public:
 
 	/// Replace the inset contents for insets which InsetCode is equal
 	/// to the passed \p inset_code.
-	void changeRefsIfUnique(docstring const & from, docstring const & to,
-		InsetCode code);
+	void changeRefsIfUnique(docstring const & from, docstring const & to);
 
 	/// get source code (latex/docbook) for some paragraphs, or all paragraphs
 	/// including preamble
 	void getSourceCode(odocstream & os, std::string const format,
-			   pit_type par_begin, pit_type par_end, OutputWhat output) const;
+			   pit_type par_begin, pit_type par_end, OutputWhat output,
+			   bool master) const;
 
 	/// Access to error list.
 	/// This method is used only for GUI visualisation of Buffer related
 	/// errors (like parsing or LateX compilation). This method is const
 	/// because modifying the returned ErrorList does not touch the document
 	/// contents.
-	ErrorList & errorList(std::string const & type);
-	ErrorList const & errorList(std::string const & type) const;
+	ErrorList & errorList(std::string const & type) const;
 
 	/// The Toc backend.
 	/// This is useful only for screen visualisation of the Buffer. This
@@ -614,27 +638,42 @@ public:
 	///
 	bool hasGuiDelegate() const;
 
-	
-
 	///
-	bool doExport(std::string const & format, bool put_in_tempdir,
+	ExportStatus doExport(std::string const & target, bool put_in_tempdir) const;
+	///
+	ExportStatus doExport(std::string const & target, bool put_in_tempdir,
+		std::string & result_file) const;
+	///
+	ExportStatus preview(std::string const & format) const;
+
+private:
+	/// target is a format name optionally followed by a space
+	/// and a destination file-name
+	ExportStatus doExport(std::string const & target, bool put_in_tempdir,
 		bool includeall, std::string & result_file) const;
 	///
-	bool doExport(std::string const & format, bool put_in_tempdir,
-		      bool includeall = false) const;
+	ExportStatus doExport(std::string const & target, bool put_in_tempdir,
+		bool includeall) const;
 	///
-	bool preview(std::string const & format, bool includeall = false) const;
-	/// mark the buffer as busy exporting something, or not
-	void setExportStatus(bool e) const;
+	ExportStatus preview(std::string const & format, bool includeall = false) const;
+	///
+	void setMathFlavor(OutputParams & op) const;
+
+public:
 	///
 	bool isExporting() const;
 
 	///
 	typedef std::vector<std::pair<Inset *, ParIterator> > References;
-	References & references(docstring const & label);
+	///
 	References const & references(docstring const & label) const;
+	///
+	void addReference(docstring const & label, Inset * inset, ParIterator it);
+	///
 	void clearReferenceCache() const;
+	///
 	void setInsetLabel(docstring const & label, InsetLabel const * il);
+	///
 	InsetLabel const * insetLabel(docstring const & label) const;
 
 	/// return a list of all used branches (also in children)
@@ -652,7 +691,7 @@ public:
 	/// do this one.
 	/// \param output: whether we are preparing for output.
 	void updateBuffer(UpdateScope scope, UpdateType utype) const;
-	/// 
+	///
 	void updateBuffer(ParIterator & parit, UpdateType utype) const;
 
 	/// Spellcheck starting from \p from.
@@ -679,8 +718,12 @@ public:
 	int charCount(bool with_blanks) const;
 
 private:
+	friend class MarkAsExporting;
+	/// mark the buffer as busy exporting something, or not
+	void setExportStatus(bool e) const;
+
 	///
-	void setMathFlavor(OutputParams & op) const;
+	References & getReferenceCache(docstring const & label);
 	/// Change name of buffer. Updates "read-only" flag.
 	void setFileName(support::FileName const & fname);
 	///
@@ -694,12 +737,15 @@ private:
 	void updateBibfilesCache(UpdateScope scope = UpdateMaster) const;
 	/// Return the list with all bibfiles in use (including bibfiles
 	/// of loaded child documents).
-	support::FileNameList const & 
+	support::FileNameList const &
 		getBibfilesCache(UpdateScope scope = UpdateMaster) const;
 	///
 	void collectChildren(ListOfBuffers & children, bool grand_children) const;
 
-	
+	/// noncopyable
+	Buffer(Buffer const &);
+	void operator=(Buffer const &);
+
 	/// Use the Pimpl idiom to hide the internals.
 	class Impl;
 	/// The pointer never changes although *pimpl_'s contents may.
@@ -707,6 +753,25 @@ private:
 };
 
 
+/// Helper class, to guarantee that the export status
+/// gets reset properly. To use, simply create a local variable:
+///    MarkAsExporting mex(bufptr);
+/// and leave the rest to us.
+class MarkAsExporting {
+public:
+	MarkAsExporting(Buffer const * buf) : buf_(buf)
+	{
+		buf_->setExportStatus(true);
+	}
+	~MarkAsExporting()
+	{
+		buf_->setExportStatus(false);
+	}
+private:
+	Buffer const * const buf_;
+};
+
+
 } // namespace lyx
 
 #endif
