@@ -16,6 +16,7 @@
 #define BUFFERPARAMS_H
 
 #include "Citation.h"
+#include "DocumentClassPtr.h"
 #include "Format.h"
 #include "LayoutModuleList.h"
 #include "OutputParams.h"
@@ -117,44 +118,54 @@ public:
 	///
 	LayoutFileIndex const & baseClassID() const;
 	/// Set the LyX layout file this document is using.
-	/// NOTE: This does not call makeDocumentClass() to update the local 
+	/// NOTE: This does not call makeDocumentClass() to update the local
 	/// DocumentClass. That needs to be done manually.
 	/// \param filename the name of the layout file
 	bool setBaseClass(std::string const & classname);
 	/// Adds the module information to the baseClass information to
 	/// create our local DocumentClass.
+	/// NOTE: This should NEVER be called externally unless one immediately goes
+	/// on to class BufferView::updateDocumentClass(). The exception, of course,
+	/// is in GuiDocument, where we use a BufferParams simply to hold a copy of
+	/// the parameters from the active Buffer.
 	void makeDocumentClass(bool const clone = false);
 	/// Returns the DocumentClass currently in use: the BaseClass as modified
 	/// by modules.
 	DocumentClass const & documentClass() const;
-	/// \return A pointer to the DocumentClass currently in use: the BaseClass 
-	/// as modified by modules. 
-	DocumentClass const * documentClassPtr() const;
+	/// \return A pointer to the DocumentClass currently in use: the BaseClass
+	/// as modified by modules.
+	DocumentClassConstPtr documentClassPtr() const;
 	/// This bypasses the baseClass and sets the textClass directly.
 	/// Should be called with care and would be better not being here,
 	/// but it seems to be needed by CutAndPaste::putClipboard().
-	void setDocumentClass(DocumentClass const * const);
+	void setDocumentClass(DocumentClassConstPtr);
 	/// List of modules in use
 	LayoutModuleList const & getModules() const { return layout_modules_; }
 	/// List of default modules the user has removed
-	std::list<std::string> const & getRemovedModules() const 
+	std::list<std::string> const & getRemovedModules() const
 			{ return removed_modules_; }
 	///
 	/// Add a module to the list of modules in use. This checks only that the
-	/// module is not already in the list, so use moduleIsCompatible first if
-	/// you want to check for compatibility.
+	/// module is not already in the list, so use layoutModuleCanBeAdeed first 
+	/// if you want to check for compatibility.
 	/// \return true if module was successfully added.
 	bool addLayoutModule(std::string const & modName);
 	/// checks to make sure module's requriements are satisfied, that it does
 	/// not conflict with already-present modules, isn't already loaded, etc.
-	bool moduleCanBeAdded(std::string const & modName) const;
+	bool layoutModuleCanBeAdded(std::string const & modName) const;
+	/// same, but for citaton modules.
+	bool citationModuleCanBeAdded(std::string const & modName) const;
 	///
-	void addRemovedModule(std::string const & modName) 
+	void addRemovedModule(std::string const & modName)
 			{ removed_modules_.push_back(modName); }
 	/// Clear the list
 	void clearLayoutModules() { layout_modules_.clear(); }
 	/// Clear the removed module list
 	void clearRemovedModules() { removed_modules_.clear(); }
+	/// Get the local layouts
+	std::string getLocalLayout(bool) const;
+	/// Set the local layouts
+	void setLocalLayout(std::string const &, bool);
 
 	/// returns \c true if the buffer contains a LaTeX document
 	bool isLatex() const;
@@ -181,10 +192,10 @@ public:
 	std::vector<std::string> backends() const;
 
 	/// List of included children (for includeonly)
-	std::list<std::string> const & getIncludedChildren() const 
+	std::list<std::string> const & getIncludedChildren() const
 			{ return included_children_; }
 	///
-	void addIncludedChildren(std::string const & child) 
+	void addIncludedChildren(std::string const & child)
 			{ included_children_.push_back(child); }
 	/// Clear the list of included children
 	void clearIncludedChildren() { included_children_.clear(); }
@@ -195,6 +206,9 @@ public:
 	/// returns the main font for the buffer (document)
 	Font const getFont() const;
 
+	/// translate quote style string to enum value
+	InsetQuotes::QuoteLanguage getQuoteStyle(std::string const qs) const;
+
 	/* these are for the PaperLayout */
 	/// the papersize
 	PAPER_SIZE papersize;
@@ -240,6 +254,8 @@ public:
 	std::string fonts_sans;
 	/// the tt font
 	std::string fonts_typewriter;
+	/// the math font
+	std::string fonts_math;
 	/// the default family (rm, sf, tt)
 	std::string fonts_default_family;
 	/// use the fonts of the OS (OpenType, True Type) directly
@@ -272,10 +288,10 @@ public:
 	IndicesList & indiceslist();
 	IndicesList const & indiceslist() const;
 	/**
-	 * The input encoding for LaTeX. This can be one of
+	 * The LyX name of the input encoding for LaTeX. This can be one of
 	 * - \c auto: find out the input encoding from the used languages
 	 * - \c default: ditto
-	 * - any encoding supported by the inputenc package
+	 * - any encoding defined in the file lib/encodings
 	 * The encoding of the LyX file is always utf8 and has nothing to
 	 * do with this setting.
 	 * The difference between \c auto and \c default is that \c auto also
@@ -293,8 +309,6 @@ public:
 	///
 	std::string preamble;
 	///
-	std::string local_layout;
-	///
 	std::string options;
 	/// use the class options defined in the layout?
 	bool use_default_options;
@@ -306,6 +320,8 @@ public:
 	std::string float_placement;
 	///
 	unsigned int columns;
+	///
+	bool justification;
 	/// parameters for the listings package
 	std::string listings_params;
 	///
@@ -342,14 +358,12 @@ public:
 		/// some ERT that needs the package)
 		package_on = 2
 	};
-	/// Whether and how to load amsmath
-	Package use_amsmath;
-	/// Whether and how to load esint
-	Package use_esint;
-	/// Whether and how to load mhchem
-	Package use_mhchem;
-	/// Whether and how to load mathdots
-	Package use_mathdots;
+	/// Whether to load a package such as amsmath or esint.
+	Package use_package(std::string const & p) const;
+	/// Set whether to load a package such as amsmath or esint.
+	void use_package(std::string const & p, Package u);
+	/// All packages that can be switched on or off
+	static std::map<std::string, std::string> const & auto_packages();
 	/// Split bibliography?
 	bool use_bibtopic;
 	/// Split the index?
@@ -398,17 +412,39 @@ public:
 	///
 	std::string const parseFontName(std::string const & name) const;
 	/// set up the document fonts
-	std::string const loadFonts(std::string const & rm,
-				     std::string const & sf, std::string const & tt,
-				     bool const & sc, bool const & osf,
-				     int const & sfscale, int const & ttscale,
-				     bool const & use_nonlatexfonts,
-				     LaTeXFeatures & features) const;
+	std::string const loadFonts(LaTeXFeatures & features) const;
 
-	/// get the appropriate cite engine (natbib handling)
-	CiteEngine citeEngine() const;
-	///
-	void setCiteEngine(CiteEngine const);
+	/// the cite engine modules
+	LayoutModuleList const & citeEngine() const
+		{ return cite_engine_; }
+	/// the type of cite engine (authoryear or numerical)
+	CiteEngineType const & citeEngineType() const
+		{ return cite_engine_type_; }
+	/// add the module to the cite engine modules
+	bool addCiteEngine(std::string const &);
+	/// add the modules to the cite engine modules
+	bool addCiteEngine(std::vector<std::string> const &);
+	/// clear the list of cite engine modules
+	void clearCiteEngine() { cite_engine_.clear(); }
+	/// set the cite engine module
+	void setCiteEngine(std::string const &);
+	/// set the cite engine modules
+	void setCiteEngine(std::vector<std::string> const &);
+	/// set the cite engine type
+	void setCiteEngineType(CiteEngineType const & engine_type)
+		{ cite_engine_type_ = engine_type; }
+
+	/// the available citation commands
+	std::vector<std::string> citeCommands() const;
+	/// the available citation styles
+	std::vector<CitationStyle> citeStyles() const;
+
+	/// the default BibTeX style file for the document
+	std::string biblio_style;
+	/// the default BibTeX style file from the TextClass
+	std::string const & defaultBiblioStyle() const;
+	/// whether the BibTeX style supports full author lists
+	bool const & fullAuthorList() const;
 
 	/// options for pdf output
 	PDFOptions & pdfoptions();
@@ -449,7 +485,7 @@ private:
 	///
 	void readPreamble(Lexer &);
 	///
-	void readLocalLayout(Lexer &);
+	void readLocalLayout(Lexer &, bool);
 	///
 	void readLanguage(Lexer &);
 	///
@@ -468,19 +504,31 @@ private:
 	typedef std::map<std::string, OutputParams::FLAVOR> DefaultFlavorCache;
 	///
 	mutable DefaultFlavorCache default_flavors_;
-	/// for use with natbib
-	CiteEngine cite_engine_;
+	/// the cite engine modules
+	LayoutModuleList cite_engine_;
+	/// the type of cite engine (authoryear or numerical)
+	CiteEngineType cite_engine_type_;
+	///
+	DocumentClassPtr doc_class_;
 	///
-	DocumentClass * doc_class_;
-	/// 
 	LayoutModuleList layout_modules_;
 	/// this is for modules that are required by the document class but that
 	/// the user has chosen not to use
 	std::list<std::string> removed_modules_;
+	/// The local layouts without the forced ones
+	std::string local_layout_;
+	/// Forced local layouts only for reading (use getLocalLayout() instead)
+	std::string forced_local_layout_;
 
 	/// the list of included children (for includeonly)
 	std::list<std::string> included_children_;
 
+	typedef std::map<std::string, Package> PackageMap;
+	/** Whether and how to load packages like amsmath, esint, mhchem,
+	 *  mathdots, stackrel, stmaryrd and undertilde.
+	 */
+	PackageMap use_packages;
+
 	/** Use the Pimpl idiom to hide those member variables that would otherwise
 	 *  drag in other header files.
 	 */
