@@ -28,6 +28,7 @@ class ErrorItem;
 class FuncRequest;
 class FuncStatus;
 class KeyMap;
+class LaTeXFonts;
 class Messages;
 class Mover;
 class Movers;
@@ -57,10 +58,6 @@ namespace frontend {
 class Application;
 }
 
-namespace graphics {
-class Previews;
-}
-
 /// initial startup
 class LyX {
 public:
@@ -147,12 +144,12 @@ private:
 	friend Messages const & getGuiMessages();
 	friend KeyMap & theTopLevelKeymap();
 	friend Movers & theMovers();
-	friend Mover const & getMover(std::string  const & fmt);
+	friend Mover const & getMover(std::string const & fmt);
 	friend void setMover(std::string const & fmt, std::string const & command);
 	friend Movers & theSystemMovers();
 	friend frontend::Application * theApp();
-	friend graphics::Previews & thePreviews();
 	friend Session & theSession();
+	friend LaTeXFonts & theLaTeXFonts();
 	friend CmdDef & theTopLevelCmdDef();
 	friend SpellChecker * theSpellChecker();
 	friend void setSpellChecker();
@@ -183,4 +180,3 @@ void dispatch(FuncRequest const & action, DispatchResult & dr);
 } // namespace lyx
 
 #endif // LYX_H
-
