@@ -20,7 +20,7 @@ namespace lyx {
 
 OutputParams::OutputParams(Encoding const * enc)
 	: flavor(LATEX), math_flavor(NotApplicable), nice(false), is_child(false),
-	  moving_arg(false), inulemcmd(false), local_font(0), master_language(0),
+	  moving_arg(false), inulemcmd(0), local_font(0), master_language(0),
 	  encoding(enc), free_spacing(false), use_babel(false), use_polyglossia(false),
 	  use_indices(false), use_japanese(false), linelen(0), depth(0),
 	  exportdata(new ExportData),
