@@ -42,7 +42,6 @@
 #include "ParagraphParameters.h"
 #include "TextClass.h"
 #include "TextMetrics.h"
-#include "VSpace.h"
 
 #include "insets/InsetCollapsable.h"
 
@@ -77,16 +76,7 @@ FontInfo Text::layoutFont(pit_type const pit) const
 		// In case the default family has been customized
 		if (layout.font.family() == INHERIT_FAMILY)
 			lf.setFamily(owner_->buffer().params().getFont().fontInfo().family());
-		// FIXME
-		// It ought to be possible here just to use Inset::getLayout() and skip
-		// the asInsetCollapsable() bit. Unfortunatley, that doesn't work right
-		// now, because Inset::getLayout() will return a default-constructed
-		// InsetLayout, and that e.g. sets the foreground color to red. So we
-		// need to do some work to make that possible.
-		InsetCollapsable const * icp = owner_->asInsetCollapsable();
-		if (!icp)
-			return lf;
-		FontInfo icf = icp->getLayout().font();
+		FontInfo icf = owner_->getLayout().font();
 		icf.realize(lf);
 		return icf;
 	}
@@ -164,10 +154,10 @@ void Text::setCharFont(pit_type pit,
 
 
 void Text::setInsetFont(BufferView const & bv, pit_type pit,
-		pos_type pos, Font const & font, bool toggleall)
+		pos_type pos, Font const & font)
 {
 	Inset * const inset = pars_[pit].getInset(pos);
-	LASSERT(inset && inset->resetFontEdit(), /**/);
+	LASSERT(inset && inset->resetFontEdit(), return);
 
 	CursorSlice::idx_type endidx = inset->nargs();
 	for (CursorSlice cs(*inset); cs.idx() != endidx; ++cs.idx()) {
@@ -177,7 +167,7 @@ void Text::setInsetFont(BufferView const & bv, pit_type pit,
 			CursorSlice cellend = cs;
 			cellend.pit() = cellend.lastpit();
 			cellend.pos() = cellend.lastpos();
-			text->setFont(bv, cs, cellend, font, toggleall);
+			text->setFont(bv, cs, cellend, font);
 		}
 	}
 }
@@ -186,7 +176,7 @@ void Text::setInsetFont(BufferView const & bv, pit_type pit,
 void Text::setLayout(pit_type start, pit_type end,
 		     docstring const & layout)
 {
-	LASSERT(start != end, /**/);
+	LASSERT(start != end, return);
 
 	Buffer const & buffer = owner_->buffer();
 	BufferParams const & bp = buffer.params();
@@ -204,7 +194,7 @@ void Text::setLayout(pit_type start, pit_type end,
 // set layout over selection and make a total rebreak of those paragraphs
 void Text::setLayout(Cursor & cur, docstring const & layout)
 {
-	LASSERT(this == cur.text(), /**/);
+	LBUFERR(this == cur.text());
 
 	pit_type start = cur.selBegin().pit();
 	pit_type end = cur.selEnd().pit() + 1;
@@ -217,8 +207,6 @@ void Text::setLayout(Cursor & cur, docstring const & layout)
 static bool changeDepthAllowed(Text::DEPTH_CHANGE type,
 			Paragraph const & par, int max_depth)
 {
-	if (par.layout().labeltype == LABEL_BIBLIO)
-		return false;
 	int const depth = par.params().depth();
 	if (type == Text::INC_DEPTH && depth < max_depth)
 		return true;
@@ -230,7 +218,7 @@ static bool changeDepthAllowed(Text::DEPTH_CHANGE type,
 
 bool Text::changeDepthAllowed(Cursor & cur, DEPTH_CHANGE type) const
 {
-	LASSERT(this == cur.text(), /**/);
+	LBUFERR(this == cur.text());
 	// this happens when selecting several cells in tabular (bug 2630)
 	if (cur.selBegin().idx() != cur.selEnd().idx())
 		return false;
@@ -250,7 +238,7 @@ bool Text::changeDepthAllowed(Cursor & cur, DEPTH_CHANGE type) const
 
 void Text::changeDepth(Cursor & cur, DEPTH_CHANGE type)
 {
-	LASSERT(this == cur.text(), /**/);
+	LBUFERR(this == cur.text());
 	pit_type const beg = cur.selBegin().pit();
 	pit_type const end = cur.selEnd().pit() + 1;
 	cur.recordUndoSelection();
@@ -275,7 +263,7 @@ void Text::changeDepth(Cursor & cur, DEPTH_CHANGE type)
 
 void Text::setFont(Cursor & cur, Font const & font, bool toggleall)
 {
-	LASSERT(this == cur.text(), /**/);
+	LASSERT(this == cur.text(), return);
 	// Set the current_font
 	// Determine basis font
 	FontInfo layoutfont;
@@ -302,15 +290,67 @@ void Text::setFont(Cursor & cur, Font const & font, bool toggleall)
 
 	// Ok, we have a selection.
 	cur.recordUndoSelection();
+	Font newfont = font;
+
+	if (toggleall) {	
+		// Toggling behaves as follows: We check the first character of the
+		// selection. If it's (say) got EMPH on, then we set to off; if off,
+		// then to on. With families and the like, we set it to INHERIT, if
+		// we already have it.
+		CursorSlice const & sl = cur.selBegin();
+		Text const & text = *sl.text();
+		Paragraph const & par = text.getPar(sl.pit());
+	
+		// get font at the position
+		Font oldfont = par.getFont(cur.bv().buffer().params(), sl.pos(),
+			text.outerFont(sl.pit()));
+		FontInfo const & oldfi = oldfont.fontInfo();
+	
+		FontInfo & newfi = newfont.fontInfo();
+	
+		FontFamily newfam = newfi.family();
+		if (newfam !=	INHERIT_FAMILY && newfam != IGNORE_FAMILY &&
+				newfam == oldfi.family())
+			newfi.setFamily(INHERIT_FAMILY);
+		
+		FontSeries newser = newfi.series();
+		if (newser == BOLD_SERIES && oldfi.series() == BOLD_SERIES)
+			newfi.setSeries(INHERIT_SERIES);
+	
+		FontShape newshp = newfi.shape();
+		if (newshp != INHERIT_SHAPE && newshp != IGNORE_SHAPE &&
+				newshp == oldfi.shape())
+			newfi.setShape(INHERIT_SHAPE);
+
+		ColorCode newcol = newfi.color();
+		if (newcol != Color_none && newcol != Color_inherit 
+		    && newcol != Color_ignore && newcol == oldfi.color())
+			newfi.setColor(Color_none);
+
+		// ON/OFF ones
+		if (newfi.emph() == FONT_TOGGLE)
+			newfi.setEmph(oldfi.emph() == FONT_OFF ? FONT_ON : FONT_OFF);
+		if (newfi.underbar() == FONT_TOGGLE)
+			newfi.setUnderbar(oldfi.underbar() == FONT_OFF ? FONT_ON : FONT_OFF);
+		if (newfi.strikeout() == FONT_TOGGLE)
+			newfi.setStrikeout(oldfi.strikeout() == FONT_OFF ? FONT_ON : FONT_OFF);
+		if (newfi.uuline() == FONT_TOGGLE)
+			newfi.setUuline(oldfi.uuline() == FONT_OFF ? FONT_ON : FONT_OFF);
+		if (newfi.uwave() == FONT_TOGGLE)
+			newfi.setUwave(oldfi.uwave() == FONT_OFF ? FONT_ON : FONT_OFF);
+		if (newfi.noun() == FONT_TOGGLE)
+			newfi.setNoun(oldfi.noun() == FONT_OFF ? FONT_ON : FONT_OFF);
+		if (newfi.number() == FONT_TOGGLE)
+			newfi.setNumber(oldfi.number() == FONT_OFF ? FONT_ON : FONT_OFF);
+	}
 
 	setFont(cur.bv(), cur.selectionBegin().top(), 
-		cur.selectionEnd().top(), font, toggleall);
+		cur.selectionEnd().top(), newfont);
 }
 
 
 void Text::setFont(BufferView const & bv, CursorSlice const & begin,
-		CursorSlice const & end, Font const & font,
-		bool toggleall)
+		CursorSlice const & end, Font const & font)
 {
 	Buffer const & buffer = bv.buffer();
 
@@ -328,11 +368,11 @@ void Text::setFont(BufferView const & bv, CursorSlice const & begin,
 		if (inset && inset->resetFontEdit()) {
 			// We need to propagate the font change to all
 			// text cells of the inset (bugs 1973, 6919).
-			setInsetFont(bv, pit, pos, font, toggleall);
+			setInsetFont(bv, pit, pos, font);
 		}
 		TextMetrics const & tm = bv.textMetrics(this);
 		Font f = tm.displayFont(pit, pos);
-		f.update(font, language, toggleall);
+		f.update(font, language);
 		setCharFont(pit, pos, f, tm.font_);
 		// font change may change language... 
 		// spell checker has to know that
@@ -343,21 +383,21 @@ void Text::setFont(BufferView const & bv, CursorSlice const & begin,
 
 bool Text::cursorTop(Cursor & cur)
 {
-	LASSERT(this == cur.text(), /**/);
+	LBUFERR(this == cur.text());
 	return setCursor(cur, 0, 0);
 }
 
 
 bool Text::cursorBottom(Cursor & cur)
 {
-	LASSERT(this == cur.text(), /**/);
+	LBUFERR(this == cur.text());
 	return setCursor(cur, cur.lastpit(), boost::prior(paragraphs().end())->size());
 }
 
 
 void Text::toggleFree(Cursor & cur, Font const & font, bool toggleall)
 {
-	LASSERT(this == cur.text(), /**/);
+	LBUFERR(this == cur.text());
 	// If the mask is completely neutral, tell user
 	if (font.fontInfo() == ignore_font && font.language() == ignore_language) {
 		// Could only happen with user style
@@ -389,7 +429,7 @@ void Text::toggleFree(Cursor & cur, Font const & font, bool toggleall)
 
 docstring Text::getStringToIndex(Cursor const & cur)
 {
-	LASSERT(this == cur.text(), /**/);
+	LBUFERR(this == cur.text());
 
 	if (cur.selection())
 		return cur.selectionAsString(false);
@@ -442,7 +482,7 @@ void Text::setLabelWidthStringToSequence(Cursor const & cur,
 
 void Text::setParagraphs(Cursor & cur, docstring arg, bool merge) 
 {
-	LASSERT(cur.text(), /**/);
+	LBUFERR(cur.text());
 
 	//FIXME UNICODE
 	string const argument = to_utf8(arg);
@@ -468,12 +508,9 @@ void Text::setParagraphs(Cursor & cur, docstring arg, bool merge)
 }
 
 
-//FIXME This is a little redundant now, but it's probably worth keeping,
-//especially if we're going to go away from using serialization internally
-//quite so much.
 void Text::setParagraphs(Cursor & cur, ParagraphParameters const & p) 
 {
-	LASSERT(cur.text(), /**/);
+	LBUFERR(cur.text());
 
 	depth_type priordepth = -1;
 	Layout priorlayout;
@@ -499,8 +536,8 @@ void Text::setParagraphs(Cursor & cur, ParagraphParameters const & p)
 // this really should just insert the inset and not move the cursor.
 void Text::insertInset(Cursor & cur, Inset * inset)
 {
-	LASSERT(this == cur.text(), /**/);
-	LASSERT(inset, /**/);
+	LBUFERR(this == cur.text());
+	LBUFERR(inset);
 	cur.paragraph().insertInset(cur.pos(), inset, cur.current_font,
 		Change(cur.buffer()->params().trackChanges
 		? Change::INSERTED : Change::UNCHANGED));
@@ -520,7 +557,7 @@ bool Text::setCursor(Cursor & cur, pit_type par, pos_type pos,
 
 void Text::setCursor(CursorSlice & cur, pit_type par, pos_type pos)
 {
-	LASSERT(par != int(paragraphs().size()), /**/);
+	LASSERT(par != int(paragraphs().size()), return);
 	cur.pit() = par;
 	cur.pos() = pos;
 
@@ -529,15 +566,15 @@ void Text::setCursor(CursorSlice & cur, pit_type par, pos_type pos)
 
 	// None of these should happen, but we're scaredy-cats
 	if (pos < 0) {
-		lyxerr << "don't like -1" << endl;
-		LASSERT(false, /**/);
+		LYXERR0("Don't like -1!");
+		LATTEST(false);
 	}
 
 	if (pos > para.size()) {
-		lyxerr << "don't like 1, pos: " << pos
+		LYXERR0("Don't like 1, pos: " << pos
 		       << " size: " << para.size()
-		       << " par: " << par << endl;
-		LASSERT(false, /**/);
+		       << " par: " << par);
+		LATTEST(false);
 	}
 }
 
@@ -545,7 +582,7 @@ void Text::setCursor(CursorSlice & cur, pit_type par, pos_type pos)
 void Text::setCursorIntern(Cursor & cur,
 			      pit_type par, pos_type pos, bool setfont, bool boundary)
 {
-	LASSERT(this == cur.text(), /**/);
+	LBUFERR(this == cur.text());
 	cur.boundary(boundary);
 	setCursor(cur.top(), par, pos);
 	if (setfont)
@@ -736,7 +773,13 @@ bool Text::cursorDownParagraph(Cursor & cur)
 {
 	bool updated = false;
 	if (cur.pit() != cur.lastpit())
-		updated = setCursor(cur, cur.pit() + 1, 0);
+		if (lyxrc.mac_like_cursor_movement)
+			if (cur.pos() == cur.lastpos())
+				updated = setCursor(cur, cur.pit() + 1, getPar(cur.pit() + 1).size());
+			else
+				updated = setCursor(cur, cur.pit(), cur.lastpos());
+		else
+			updated = setCursor(cur, cur.pit() + 1, 0);
 	else
 		updated = setCursor(cur, cur.pit(), cur.lastpos());
 	return updated;
@@ -880,7 +923,7 @@ bool Text::deleteEmptyParagraphMechanism(Cursor & cur,
 
 void Text::deleteEmptyParagraphMechanism(pit_type first, pit_type last, bool trackChanges)
 {
-	LASSERT(first >= 0 && first <= last && last < (int) pars_.size(), /**/);
+	LASSERT(first >= 0 && first <= last && last < (int) pars_.size(), return);
 
 	for (pit_type pit = first; pit <= last; ++pit) {
 		Paragraph & par = pars_[pit];
