@@ -309,7 +309,7 @@ bool LyXDataSocket::connected() const
 // Returns true if there was a complete line to input
 // A line is of the form <key>:<value>
 //   A line not of this form will not be passed
-// The line read is splitted and stored in 'key' and 'value'
+// The line read is split and stored in 'key' and 'value'
 bool LyXDataSocket::readln(string & line)
 {
 	int const charbuf_size = 100;
@@ -424,6 +424,9 @@ bool CmdLineParser::parse(int argc, char * argv[])
 
 namespace cmdline {
 
+    docstring mainTmp(from_ascii("/tmp"));
+
+
 void usage()
 {
 	cerr <<
@@ -437,10 +440,12 @@ void usage()
 	  "  -n name       set client name\n"
 	  "  -h name       display this help end exit\n"
 	  "If -a is not used, lyxclient will use the arguments of -t and -p to look for\n"
-	  "a running lyx. If -t is not set, 'directory' defaults to /tmp. If -p is set,\n"
+	  "a running lyx. If -t is not set, 'directory' defaults to the system directory. If -p is set,\n"
 	  "lyxclient will connect only to a lyx with the specified pid. Options -c and -g\n"
 	  "cannot be set simultaneoulsly. If no -c or -g options are given, lyxclient\n"
-	  "will read commands from standard input and disconnect when command read is BYE:"
+	  "will read commands from standard input and disconnect when command read is BYE:\n"
+      "\n"
+      "System directory is: " << to_utf8(cmdline::mainTmp)
 	   << endl;
 }
 
@@ -457,7 +462,7 @@ docstring clientName =
 
 int n(vector<docstring> const & arg)
 {
-	if (arg.size() < 1) {
+	if (arg.empty()) {
 		cerr << "lyxclient: The option -n requires 1 argument."
 		     << endl;
 		return -1;
@@ -472,7 +477,7 @@ docstring singleCommand;
 
 int c(vector<docstring> const & arg)
 {
-	if (arg.size() < 1) {
+	if (arg.empty()) {
 		cerr << "lyxclient: The option -c requires 1 argument."
 		     << endl;
 		return -1;
@@ -502,7 +507,7 @@ docstring serverAddress;
 
 int a(vector<docstring> const & arg)
 {
-	if (arg.size() < 1) {
+	if (arg.empty()) {
 		cerr << "lyxclient: The option -a requires 1 argument."
 		     << endl;
 		return -1;
@@ -513,12 +518,11 @@ int a(vector<docstring> const & arg)
 }
 
 
-docstring mainTmp(from_ascii("/tmp"));
 
 
 int t(vector<docstring> const & arg)
 {
-	if (arg.size() < 1) {
+	if (arg.empty()) {
 		cerr << "lyxclient: The option -t requires 1 argument."
 		     << endl;
 		return -1;
@@ -533,7 +537,7 @@ string serverPid; // Init to empty string
 
 int p(vector<docstring> const & arg)
 {
-	if (arg.size() < 1) {
+	if (arg.empty()) {
 		cerr << "lyxclient: The option -p requires 1 argument."
 		     << endl;
 		return -1;
@@ -552,10 +556,16 @@ int main(int argc, char * argv[])
 	using namespace lyx;
 	lyxerr.setStream(cerr);
 
+
+    // Set defaults
 	char const * const lyxsocket = getenv("LYXSOCKET");
 	if (lyxsocket)
 		cmdline::serverAddress = from_local8bit(lyxsocket);
 
+    // Default temporary
+    cmdline::mainTmp = FileName::tempPath().absoluteFilePath();
+
+    // Command line builder
 	CmdLineParser args;
 	args.helper["-h"] = cmdline::h;
 	args.helper["-c"] = cmdline::c;
