@@ -19,11 +19,15 @@
 #include "qt_helpers.h"
 #include "Language.h"
 
+#include "Buffer.h"
 #include "BufferParams.h"
 #include "BufferList.h"
+#include "BufferView.h"
+#include "Text.h"
 #include "TextClass.h"
 #include "Cursor.h"
 #include "FuncRequest.h"
+#include "LyX.h"
 #include "lyxfind.h"
 
 #include "frontends/alert.h"
@@ -65,6 +69,18 @@ FindAndReplaceWidget::FindAndReplaceWidget(GuiView & view)
 }
 
 
+void FindAndReplaceWidget::dockLocationChanged(Qt::DockWidgetArea area)
+{
+       if (area == Qt::RightDockWidgetArea || area == Qt::LeftDockWidgetArea) {
+               dynamicLayoutBasic_->setDirection(QBoxLayout::TopToBottom);
+               dynamicLayoutAdvanced_->setDirection(QBoxLayout::TopToBottom);
+       } else {
+               dynamicLayoutBasic_->setDirection(QBoxLayout::LeftToRight);
+               dynamicLayoutAdvanced_->setDirection(QBoxLayout::LeftToRight);
+       }
+}
+
+
 bool FindAndReplaceWidget::eventFilter(QObject * obj, QEvent * event)
 {
 	if (event->type() != QEvent::KeyPress
@@ -147,7 +163,7 @@ static vector<string> const & allManualsFiles()
 }
 
 
-/** Switch p_buf to point to next document buffer.
+/** Switch buf to point to next document buffer.
  **
  ** Return true if restarted from master-document buffer.
  **/
@@ -157,7 +173,7 @@ static bool nextDocumentBuffer(Buffer * & buf)
 	LYXERR(Debug::FIND, "children.size()=" << children.size());
 	ListOfBuffers::const_iterator it =
 		find(children.begin(), children.end(), buf);
-	LASSERT(it != children.end(), /**/);
+	LASSERT(it != children.end(), return false);
 	++it;
 	if (it == children.end()) {
 		buf = *children.begin();
@@ -178,7 +194,7 @@ static bool prevDocumentBuffer(Buffer * & buf)
 	LYXERR(Debug::FIND, "children.size()=" << children.size());
 	ListOfBuffers::const_iterator it =
 		find(children.begin(), children.end(), buf);
-	LASSERT(it != children.end(), /**/)
+	LASSERT(it != children.end(), return false)
 	if (it == children.begin()) {
 		it = children.end();
 		--it;
@@ -294,6 +310,8 @@ bool FindAndReplaceWidget::findAndReplaceScope(FindAndReplaceOptions & opt, bool
 	oss << opt;
 	FuncRequest cmd(LFUN_WORD_FINDADV, from_utf8(oss.str()));
 
+	view_.message(_("Advanced search in progress (press ESC to cancel) . . ."));
+	theApp()->startLongOperation();
 	view_.setBusy(true);
 	if (opt.scope == FindAndReplaceOptions::S_ALL_MANUALS) {
 		vector<string> const & v = allManualsFiles();
@@ -301,7 +319,9 @@ bool FindAndReplaceWidget::findAndReplaceScope(FindAndReplaceOptions & opt, bool
 			FileName const & fname = FileName(*v.begin());
 			if (!theBufferList().exists(fname)) {
 				guiApp->currentView()->setBusy(false);
+				theApp()->stopLongOperation();
 				guiApp->currentView()->loadDocument(fname, false);
+				theApp()->startLongOperation();
 				guiApp->currentView()->setBusy(true);
 			}
 			buf = theBufferList().getBuffer(fname);
@@ -327,10 +347,19 @@ bool FindAndReplaceWidget::findAndReplaceScope(FindAndReplaceOptions & opt, bool
 			if (replace_all)
 				continue;
 			view_.setBusy(false);
+			theApp()->stopLongOperation();
 			return true;
 		} else if (replace_all)
 			bv->clearSelection();
 
+		if (theApp()->longOperationCancelled()) {
+			// Search aborted by user
+			view_.message(_("Advanced search cancelled by user"));
+			view_.setBusy(false);
+			theApp()->stopLongOperation();
+			return false;
+		}
+
 		// No match found in current buffer (however old selection might have been replaced)
 		// select next buffer in scope, if any
 		bool const prompt = nextPrevBuffer(buf, opt);
@@ -341,9 +370,11 @@ bool FindAndReplaceWidget::findAndReplaceScope(FindAndReplaceOptions & opt, bool
 				break;
 			docstring q = getQuestionString(opt);
 			view_.setBusy(false);
+			theApp()->stopLongOperation();
 			wrap_answer = frontend::Alert::prompt(
 				_("Wrap search?"), q,
 				0, 1, _("&Yes"), _("&No"));
+			theApp()->startLongOperation();
 			view_.setBusy(true);
 			if (wrap_answer == 1)
 				break;
@@ -373,6 +404,7 @@ bool FindAndReplaceWidget::findAndReplaceScope(FindAndReplaceOptions & opt, bool
 		cur_orig.pos() = cur_orig.lastpos();
 	bv->cursor().setCursor(cur_orig);
 	view_.setBusy(false);
+	theApp()->stopLongOperation();
 	return false;
 }
 
@@ -406,7 +438,13 @@ bool FindAndReplaceWidget::findAndReplace(
 	else if (AllManualsRB->isChecked())
 		scope = FindAndReplaceOptions::S_ALL_MANUALS;
 	else
-		LASSERT(false, /**/);
+		LATTEST(false);
+
+	FindAndReplaceOptions::SearchRestriction restr =
+		OnlyMaths->isChecked()
+			? FindAndReplaceOptions::R_ONLY_MATHS
+			: FindAndReplaceOptions::R_EVERYTHING;
+
 	LYXERR(Debug::FIND, "FindAndReplaceOptions: "
 	       << "find_buf_name=" << find_buf_name
 	       << ", casesensitiv=" << casesensitive
@@ -416,10 +454,12 @@ bool FindAndReplaceWidget::findAndReplace(
 	       << ", ignoreformat=" << ignoreformat
 	       << ", repl_buf_name" << repl_buf_name
 	       << ", keep_case=" << keep_case
-	       << ", scope=" << scope);
+	       << ", scope=" << scope
+	       << ", restr=" << restr);
+
 	FindAndReplaceOptions opt(find_buf_name, casesensitive, matchword,
 				  !backwards, expandmacros, ignoreformat,
-				  repl_buf_name, keep_case, scope);
+				  repl_buf_name, keep_case, scope, restr);
 	return findAndReplaceScope(opt, replace_all);
 }
 
@@ -487,7 +527,7 @@ static void copy_params(BufferView const & bv, BufferView & dest_bv) {
 	Buffer & dest_buf = dest_bv.buffer();
 	dest_buf.params().setLanguage(lang);
 	dest_buf.params().setBaseClass(doc_class);
-	dest_buf.params().makeDocumentClass();
+	dest_bv.makeDocumentClass();
 	dest_bv.cursor().current_font.setLanguage(doc_bp.language);
 }
 
@@ -525,6 +565,12 @@ bool FindAndReplaceWidget::initialiseParams(std::string const & /*params*/)
 }
 
 
+void FindAndReplace::updateView()
+{
+	widget_->updateGUI();
+}
+
+
 FindAndReplace::FindAndReplace(GuiView & parent,
 		Qt::DockWidgetArea area, Qt::WindowFlags flags)
 	: DockView(parent, "findreplaceadv", qt_("Advanced Find and Replace"),
@@ -533,6 +579,9 @@ FindAndReplace::FindAndReplace(GuiView & parent,
 	widget_ = new FindAndReplaceWidget(parent);
 	setWidget(widget_);
 	setFocusProxy(widget_);
+
+	connect(this, SIGNAL(dockLocationChanged(Qt::DockWidgetArea)),
+               widget_, SLOT(dockLocationChanged(Qt::DockWidgetArea)));
 }
 
 
@@ -549,6 +598,16 @@ bool FindAndReplace::initialiseParams(std::string const & params)
 }
 
 
+void FindAndReplaceWidget::updateGUI()
+{
+	bool replace_enabled = view_.documentBufferView()
+		&& !view_.documentBufferView()->buffer().isReadonly();
+	replace_work_area_->setEnabled(replace_enabled);
+	replacePB->setEnabled(replace_enabled);
+	replaceallPB->setEnabled(replace_enabled);
+}
+
+
 Dialog * createGuiSearchAdv(GuiView & lv)
 {
 	FindAndReplace * gui = new FindAndReplace(lv, Qt::RightDockWidgetArea);
