@@ -22,6 +22,7 @@
 #include "LyX.h"
 #include "LyXRC.h"
 #include "Paragraph.h"
+#include "qt_helpers.h"
 #include "version.h"
 
 #include "support/lassert.h"
@@ -29,6 +30,7 @@
 
 #include <QApplication>
 #include <QHeaderView>
+#include <QKeyEvent>
 #include <QPainter>
 #include <QPixmapCache>
 #include <QScrollBar>
@@ -89,9 +91,10 @@ public:
 	~GuiCompletionModel() { delete list_; }
 	///
 	void setList(CompletionList const * l) {
+		beginResetModel();
 		delete list_;
 		list_ = l;
-		reset();
+		endResetModel();
 	}
 	///
 	bool sorted() const
@@ -284,19 +287,19 @@ bool GuiCompleter::inlineVisible() const
 }
 
 
-void GuiCompleter::updateVisibility(Cursor & cur, bool start, bool keep, bool cursorInView)
+void GuiCompleter::updateVisibility(Cursor & cur, bool start, bool keep)
 {
 	// parameters which affect the completion
 	bool moved = cur != old_cursor_;
 	if (moved)
 		old_cursor_ = cur;
 
-	bool possiblePopupState = popupPossible(cur) && cursorInView;
-	bool possibleInlineState = inlinePossible(cur) && cursorInView;
+	bool const possiblePopupState = popupPossible(cur);
+	bool const possibleInlineState = inlinePossible(cur);
 
 	// we moved or popup state is not ok for popup?
 	if ((moved && !keep) || !possiblePopupState)
-		hidePopup(cur);
+		hidePopup();
 
 	// we moved or inline state is not ok for inline completion?
 	if ((moved && !keep) || !possibleInlineState)
@@ -340,7 +343,7 @@ void GuiCompleter::updateVisibility(bool start, bool keep)
 }
 
 
-void GuiCompleter::updatePrefix(Cursor & cur)
+void GuiCompleter::updatePrefix(Cursor const & cur)
 {
 	// get new prefix. Do nothing if unchanged
 	QString newPrefix = toqstr(cur.inset().completionPrefix(cur));
@@ -379,7 +382,7 @@ void GuiCompleter::updatePrefix(Cursor & cur)
 }
 
 
-void GuiCompleter::updateInline(Cursor & cur, QString const & completion)
+void GuiCompleter::updateInline(Cursor const & cur, QString const & completion)
 {
 	if (!cur.inset().inlineCompletionSupported(cur))
 		return;
@@ -400,7 +403,7 @@ void GuiCompleter::updateInline(Cursor & cur, QString const & completion)
 }
 
 
-void GuiCompleter::updatePopup(Cursor & cur)
+void GuiCompleter::updatePopup(Cursor const & cur)
 {
 	if (!cur.inset().completionSupported(cur))
 		return;
@@ -443,8 +446,8 @@ void GuiCompleter::asyncUpdatePopup()
 	// has a bad memory about it and we have to tell him again and again.
 	QTreeView * listView = static_cast<QTreeView *>(popup());
 	listView->header()->setStretchLastSection(false);
-	listView->header()->setResizeMode(0, QHeaderView::Stretch);
-	listView->header()->setResizeMode(1, QHeaderView::Fixed);
+	setSectionResizeMode(listView->header(), 0, QHeaderView::Stretch);
+	setSectionResizeMode(listView->header(), 1, QHeaderView::Fixed);
 	listView->header()->resizeSection(1, 22);
 	
 	// show/update popup
@@ -458,7 +461,7 @@ void GuiCompleter::updateAvailability()
 	// visible yet, i.e. especially if automatic completion is disabled.
 	if (inlineVisible() || popupVisible())
 		return;
-	Cursor & cur = gui_->bufferView().cursor();
+	Cursor const & cur = gui_->bufferView().cursor();
 	if (!popupPossible(cur) && !inlinePossible(cur))
 		return;
 	
@@ -466,7 +469,7 @@ void GuiCompleter::updateAvailability()
 }
 	
 
-void GuiCompleter::updateModel(Cursor & cur, bool popupUpdate, bool inlineUpdate)
+void GuiCompleter::updateModel(Cursor const & cur, bool popupUpdate, bool inlineUpdate)
 {
 	// value which should be kept selected
 	QString old = currentCompletion();
@@ -520,7 +523,7 @@ void GuiCompleter::updateModel(Cursor & cur, bool popupUpdate, bool inlineUpdate
 }
 
 
-void GuiCompleter::showPopup(Cursor & cur)
+void GuiCompleter::showPopup(Cursor const & cur)
 {
 	if (!popupPossible(cur))
 		return;
@@ -529,26 +532,6 @@ void GuiCompleter::showPopup(Cursor & cur)
 }
 
 
-void GuiCompleter::hidePopup(Cursor &)
-{
-	popupVisible_ = false;
-
-	if (popup_timer_.isActive())
-		popup_timer_.stop();
-
-	// hide popup asynchronously because we might be here inside of
-	// LFUN dispatchers. Hiding a popup can trigger a focus event on the 
-	// workarea which then redisplays the cursor. But the metrics are not
-	// yet up to date such that the coord cache has not all insets yet. The
-	// cursorPos methods would triggers asserts in the coord cache then.
-	QTimer::singleShot(0, this, SLOT(asyncHidePopup()));
-	
-	// mark that the asynchronous part will reset the model
-	if (!inlineVisible())
-		modelActive_ = false;
-}
-
-
 void GuiCompleter::asyncHidePopup()
 {
 	popup()->hide();
@@ -557,7 +540,7 @@ void GuiCompleter::asyncHidePopup()
 }
 
 
-void GuiCompleter::showInline(Cursor & cur)
+void GuiCompleter::showInline(Cursor const & cur)
 {
 	if (!inlinePossible(cur))
 		return;
@@ -566,7 +549,7 @@ void GuiCompleter::showInline(Cursor & cur)
 }
 
 
-void GuiCompleter::hideInline(Cursor & cur)
+void GuiCompleter::hideInline(Cursor const & cur)
 {
 	gui_->bufferView().setInlineCompletion(cur, DocIterator(cur.buffer()), docstring());
 	inlineVisible_ = false;
@@ -620,14 +603,21 @@ void GuiCompleter::showInline()
 
 void GuiCompleter::hidePopup()
 {
-	Cursor cur = gui_->bufferView().cursor();
-	cur.screenUpdateFlags(Update::None);
+	popupVisible_ = false;
+
+	if (popup_timer_.isActive())
+		popup_timer_.stop();
+
+	// hide popup asynchronously because we might be here inside of
+	// LFUN dispatchers. Hiding a popup can trigger a focus event on the
+	// workarea which then redisplays the cursor. But the metrics are not
+	// yet up to date such that the coord cache has not all insets yet. The
+	// cursorPos methods would triggers asserts in the coord cache then.
+	QTimer::singleShot(0, this, SLOT(asyncHidePopup()));
 	
-	hidePopup(cur);
-	
-	// redraw if needed
-	if (cur.result().screenUpdate())
-		gui_->bufferView().processUpdateFlags(cur.result().screenUpdate());
+	// mark that the asynchronous part will reset the model
+	if (!inlineVisible())
+		modelActive_ = false;
 }
 
 
@@ -694,7 +684,7 @@ void GuiCompleter::tab()
 		cur.inset().insertCompletion(cur, docstring(), true);
 		
 		// hide popup and inline completion
-		hidePopup(cur);
+		hidePopup();
 		hideInline(cur);
 		updateVisibility(false, false);
 		cur.endUndoGroup();
@@ -798,7 +788,8 @@ void GuiCompleter::setCurrentCompletion(QString const & s)
 			i = n;
 		else
 			i = l;
-		LASSERT(i <= n, /**/);
+		// we can try to recover
+		LASSERT(i <= n, i = 0);
 	}
 
 	// select the first if none was found
@@ -887,7 +878,7 @@ void GuiCompleter::popupActivated(const QString & completion)
 	docstring prefix = cur.inset().completionPrefix(cur);
 	docstring postfix = qstring_to_ucs4(completion.mid(prefix.length()));
 	cur.inset().insertCompletion(cur, postfix, true);
-	hidePopup(cur);
+	hidePopup();
 	hideInline(cur);
 	
 	if (cur.result().screenUpdate())
