@@ -31,6 +31,7 @@
 
 #include "insets/InsetTabular.h"
 
+#include "support/convert.h"
 #include "support/debug.h"
 
 #include <QCheckBox>
@@ -126,7 +127,7 @@ GuiTabular::GuiTabular(QWidget * parent)
 		this, SLOT(checkEnabled()));
 	connect(captionStatusCB, SIGNAL(clicked()),
 		this, SLOT(checkEnabled()));
-	connect(specialAlignmentED, SIGNAL(editingFinished()),
+	connect(specialAlignmentED, SIGNAL(textEdited(const QString &)),
 		this, SLOT(checkEnabled()));
 	connect(columnWidthED, SIGNAL(textEdited(const QString &)),
 		this, SLOT(checkEnabled()));
@@ -142,8 +143,12 @@ GuiTabular::GuiTabular(QWidget * parent)
 		this, SLOT(checkEnabled()));
 	connect(rotateTabularCB, SIGNAL(clicked()),
 		this, SLOT(checkEnabled()));
+	connect(rotateTabularAngleSB, SIGNAL(valueChanged(int)),
+		this, SLOT(checkEnabled()));
 	connect(rotateCellCB, SIGNAL(clicked()),
 		this, SLOT(checkEnabled()));
+	connect(rotateCellAngleSB, SIGNAL(valueChanged(int)),
+		this, SLOT(checkEnabled()));
 	connect(TableAlignCO, SIGNAL(activated(int)),
 		this, SLOT(checkEnabled()));
 	connect(longTabularCB, SIGNAL(clicked()),
@@ -199,6 +204,7 @@ void GuiTabular::checkEnabled()
 	// if there is a LaTeX argument, the width and alignment will be overwritten
 	// therefore disable them in this case
 	columnWidthED->setEnabled(specialAlignmentED->text().isEmpty());
+	columnWidthUnitLC->setEnabled(specialAlignmentED->text().isEmpty());
 	// if the column has a width, multirows are always left-aligned
 	// therefore disable hAlignCB in this case
 	hAlignCO->setEnabled(!(multirowCB->isChecked()
@@ -222,6 +228,9 @@ void GuiTabular::checkEnabled()
 	tabularWidthED->setEnabled(setwidth);
 	tabularWidthUnitLC->setEnabled(setwidth);
 
+	rotateTabularAngleSB->setEnabled(rotateTabularCB->isChecked());
+	rotateCellAngleSB->setEnabled(rotateCellCB->isChecked());
+
 	bool const enable_valign =
 		!multirowCB->isChecked()
 		&& !widgetsToLength(columnWidthED, columnWidthUnitLC).empty()
@@ -252,7 +261,10 @@ void GuiTabular::checkEnabled()
 	// depending on the cursor position is very very unintuitive...
 	// We need some edit boxes to show which rows are header/footer/etc
 	// without having to move the cursor first.
-	headerStatusCB->setEnabled(longtabular);
+	headerStatusCB->setEnabled(longtabular
+		&& (headerStatusCB->isChecked() ?
+		    funcEnabled(Tabular::UNSET_LTHEAD) :
+		    funcEnabled(Tabular::SET_LTHEAD)));
 	headerBorderAboveCB->setEnabled(longtabular
 		&& headerStatusCB->isChecked());
 	headerBorderBelowCB->setEnabled(longtabular
@@ -264,14 +276,19 @@ void GuiTabular::checkEnabled()
 	// check if setting a first header is allowed
 	// additionally check firstheaderNoContentsCB because when this is
 	// the case a first header makes no sense
-	firstheaderStatusCB->setEnabled(funcEnabled(Tabular::SET_LTFIRSTHEAD)
+	firstheaderStatusCB->setEnabled((firstheaderStatusCB->isChecked() ?
+		   funcEnabled(Tabular::UNSET_LTFIRSTHEAD) :
+		   funcEnabled(Tabular::SET_LTFIRSTHEAD))
 		&& longtabular && !firstheaderNoContentsCB->isChecked());
 	firstheaderBorderAboveCB->setEnabled(longtabular
 		&& firstheaderStatusCB->isChecked());
 	firstheaderBorderBelowCB->setEnabled(longtabular
 		&& firstheaderStatusCB->isChecked());
 
-	footerStatusCB->setEnabled(longtabular);
+	footerStatusCB->setEnabled(longtabular
+		&& (footerStatusCB->isChecked() ?
+		    funcEnabled(Tabular::UNSET_LTFOOT) :
+		    funcEnabled(Tabular::SET_LTFOOT)));
 	footerBorderAboveCB->setEnabled(longtabular
 		&& footerBorderAboveCB->isChecked());
 	footerBorderBelowCB->setEnabled(longtabular
@@ -283,7 +300,9 @@ void GuiTabular::checkEnabled()
 	// check if setting a last footer is allowed
 	// additionally check lastfooterNoContentsCB because when this is
 	// the case a last footer makes no sense
-	lastfooterStatusCB->setEnabled(funcEnabled(Tabular::SET_LTLASTFOOT)
+	lastfooterStatusCB->setEnabled((lastfooterStatusCB->isChecked() ?
+		   funcEnabled(Tabular::UNSET_LTLASTFOOT) :
+		   funcEnabled(Tabular::SET_LTLASTFOOT))
 		&& longtabular && !lastfooterNoContentsCB->isChecked());
 	lastfooterBorderAboveCB->setEnabled(longtabular
 		&& lastfooterBorderAboveCB->isChecked());
@@ -551,16 +570,18 @@ docstring GuiTabular::dialogToParams() const
 		setParam(param_str, Tabular::SET_MULTIROW);
 	else
 		setParam(param_str, Tabular::UNSET_MULTIROW);
-	//
+	// store the table rotation angle
+	string const tabular_angle = convert<string>(rotateTabularAngleSB->value());
 	if (rotateTabularCB->isChecked())
-		setParam(param_str, Tabular::SET_ROTATE_TABULAR);
+		setParam(param_str, Tabular::SET_ROTATE_TABULAR, tabular_angle);
 	else
-		setParam(param_str, Tabular::UNSET_ROTATE_TABULAR);
-	//
+		setParam(param_str, Tabular::UNSET_ROTATE_TABULAR, tabular_angle);
+	// store the cell rotation angle
+	string const cell_angle = convert<string>(rotateCellAngleSB->value());
 	if (rotateCellCB->isChecked())
-		setParam(param_str, Tabular::SET_ROTATE_CELL);
+		setParam(param_str, Tabular::SET_ROTATE_CELL, cell_angle);
 	else
-		setParam(param_str, Tabular::UNSET_ROTATE_CELL);
+		setParam(param_str, Tabular::UNSET_ROTATE_CELL, cell_angle);
 	//
 	if (longTabularCB->isChecked())
 		setParam(param_str, Tabular::SET_LONGTABULAR);
@@ -698,8 +719,17 @@ void GuiTabular::paramsToDialog(Inset const * inset)
 	bool const multirow = tabular.isMultiRow(cell);
 	multirowCB->setChecked(multirow);
 
-	rotateCellCB->setChecked(tabular.getRotateCell(cell));
-	rotateTabularCB->setChecked(tabular.rotate);
+	rotateCellCB->setChecked(tabular.getRotateCell(cell) != 0);
+	if (rotateCellCB->isChecked()) {
+		if (tabular.getRotateCell(cell) != 0)
+			rotateCellAngleSB->setValue(tabular.getRotateCell(cell));
+		else
+			rotateCellAngleSB->setValue(90);
+	}
+
+	rotateTabularCB->setChecked(tabular.rotate != 0);
+	if (rotateTabularCB->isChecked())
+		rotateTabularAngleSB->setValue(tabular.rotate != 0 ? tabular.rotate : 90);
 
 	longTabularCB->setChecked(tabular.is_long_tabular);
 
