@@ -34,7 +34,7 @@
 #include "support/lyxalgo.h"
 #include "support/os.h"
 #include "support/Package.h"
-#include "support/Path.h"
+#include "support/PathChanger.h"
 #include "support/Systemcall.h"
 
 #include <QCheckBox>
@@ -75,6 +75,23 @@ FileName imageLibFileSearch(QString & dir, QString const & name,
 	return fn;
 }
 
+namespace {
+
+double locstringToDouble(QString const str)
+{
+	QLocale loc;
+	bool ok;
+	double res = loc.toDouble(str, &ok);
+	if (!ok) {
+		// Fall back to C
+		QLocale c(QLocale::C);
+		res = c.toDouble(str);
+	}
+	return res;
+}
+
+} // namespace anon
+
 
 namespace frontend {
 
@@ -90,7 +107,7 @@ string widgetsToLength(QLineEdit const * input, LengthCombo const * combo)
 
 	Length::UNIT const unit = combo->currentLengthItem();
 
-	return Length(length.trimmed().toDouble(), unit).asString();
+	return Length(locstringToDouble(length.trimmed()), unit).asString();
 }
 
 
@@ -113,17 +130,23 @@ Length widgetsToLength(QLineEdit const * input, QComboBox const * combo)
 		}
 	}
 
-	return Length(length.trimmed().toDouble(), unit);
+	return Length(locstringToDouble(length.trimmed()), unit);
 }
 
 
 void lengthToWidgets(QLineEdit * input, LengthCombo * combo,
-                     Length const & len, Length::UNIT /*defaultUnit*/)
+	Length const & len, Length::UNIT /*defaultUnit*/)
 {
-	combo->setCurrentItem(len.unit());
-	QLocale loc;
-	loc.setNumberOptions(QLocale::OmitGroupSeparator);
-	input->setText(loc.toString(Length(len).value()));
+	if (len.empty()) {
+		// no length (UNIT_NONE)
+		combo->setCurrentItem(Length::defaultUnit());
+		input->setText("");
+	} else {
+		combo->setCurrentItem(len.unit());
+		QLocale loc;
+		loc.setNumberOptions(QLocale::OmitGroupSeparator);
+		input->setText(loc.toString(Length(len).value()));
+	}
 }
 
 
@@ -156,18 +179,14 @@ double widgetToDouble(QLineEdit const * input)
 	QString const text = input->text();
 	if (text.isEmpty())
 		return 0.0;
-	
-	return text.trimmed().toDouble();
+
+	return locstringToDouble(text.trimmed());
 }
 
 
 string widgetToDoubleStr(QLineEdit const * input)
 {
-	QString const text = input->text();
-	if (text.isEmpty())
-		return string();
-	
-	return convert<string>(text.trimmed().toDouble());
+	return convert<string>(widgetToDouble(input));
 }
 
 
@@ -196,6 +215,23 @@ void setValid(QWidget * widget, bool valid)
 	}
 }
 
+/// wrapper to hide the change of method name to setSectionResizeMode
+void setSectionResizeMode(QHeaderView * view,
+    int logicalIndex, QHeaderView::ResizeMode mode) {
+#if (QT_VERSION >= 0x050000)
+	view->setSectionResizeMode(logicalIndex, mode);
+#else
+	view->setResizeMode(logicalIndex, mode);
+#endif
+}
+
+void setSectionResizeMode(QHeaderView * view, QHeaderView::ResizeMode mode) {
+#if (QT_VERSION >= 0x050000)
+	view->setSectionResizeMode(mode);
+#else
+	view->setResizeMode(mode);
+#endif
+}
 } // namespace frontend
 
 QString const qt_(char const * str, const char *)
@@ -210,20 +246,27 @@ QString const qt_(string const & str)
 }
 
 
-void rescanTexStyles()
+QString const qt_(QString const & qstr)
+{
+	return toqstr(_(fromqstr(qstr)));
+}
+
+
+void rescanTexStyles(string const & arg)
 {
 	// Run rescan in user lyx directory
 	PathChanger p(package().user_support());
-	FileName const command = support::libFileSearch("scripts", "TeXFiles.py");
+	FileName const prog = support::libFileSearch("scripts", "TeXFiles.py");
 	Systemcall one;
-	int const status = one.startscript(Systemcall::Wait,
-			os::python() + ' ' +
-			quoteName(command.toFilesystemEncoding()));
+	string const command = os::python() + ' ' +
+	    quoteName(prog.toFilesystemEncoding()) + ' ' +
+	    arg;
+	int const status = one.startscript(Systemcall::Wait, command);
 	if (status == 0)
 		return;
 	// FIXME UNICODE
 	frontend::Alert::error(_("Could not update TeX information"),
-		bformat(_("The script `%1$s' failed."), from_utf8(command.absFileName())));
+		bformat(_("The script `%1$s' failed."), from_utf8(prog.absFileName())));
 }
 
 
@@ -347,10 +390,11 @@ QString makeAbsPath(QString const & relpath, QString const & base)
 static string const convert_brace_glob(string const & glob)
 {
 	// Matches " *.{abc,def,ghi}", storing "*." as group 1 and
-	// "abc,def,ghi" as group 2.
-	static lyx::regex const glob_re(" *([^ {]*)\\{([^ }]+)\\}");
-	// Matches "abc" and "abc,", storing "abc" as group 1.
-	static lyx::regex const block_re("([^,}]+),?");
+	// "abc,def,ghi" as group 2, while allowing spaces in group 2.
+	static lyx::regex const glob_re(" *([^ {]*)\\{([^}]+)\\}");
+	// Matches "abc" and "abc,", storing "abc" as group 1,
+	// while ignoring surrounding spaces.
+	static lyx::regex const block_re(" *([^ ,}]+) *,? *");
 
 	string pattern;
 
@@ -423,7 +467,7 @@ QString Filter::toString() const
 {
 	QString s;
 
-	bool const has_description = desc_.empty();
+	bool const has_description = !desc_.empty();
 
 	if (has_description) {
 		s += toqstr(desc_);
@@ -508,9 +552,9 @@ FileFilterList::FileFilterList(docstring const & qt_style_filter)
 
 void FileFilterList::parse_filter(string const & filter)
 {
-	// Matches "TeX documents (*.tex)",
-	// storing "TeX documents " as group 1 and "*.tex" as group 2.
-	static lyx::regex const filter_re("([^(]*)\\(([^)]+)\\) *$");
+	// Matches "TeX documents (plain) (*.tex)",
+	// storing "TeX documents (plain) " as group 1 and "*.tex" as group 2.
+	static lyx::regex const filter_re("(.*)\\(([^()]+)\\) *$");
 
 	match_results<string::const_iterator> what;
 	if (!lyx::regex_search(filter, what, filter_re)) {
@@ -552,27 +596,29 @@ QString guiName(string const & type, BufferParams const & bp)
 	if (type == "child")
 		return qt_("Child Documents");
 	if (type == "graphics")
-		return qt_("List of Graphics");
+		return qt_("Graphics");
 	if (type == "equation")
-		return qt_("List of Equations");
+		return qt_("Equations");
 	if (type == "footnote")
-		return qt_("List of Footnotes");
+		return qt_("Footnotes");
 	if (type == "listing")
-		return qt_("List of Listings");
+		return qt_("Listings");
 	if (type == "index")
-		return qt_("List of Index Entries");
+		return qt_("Index Entries");
 	if (type == "marginalnote")
-		return qt_("List of Marginal notes");
+		return qt_("Marginal notes");
+	if (type == "nomencl")
+		return qt_("Nomenclature Entries");
 	if (type == "note")
-		return qt_("List of Notes");
+		return qt_("Notes");
 	if (type == "citation")
-		return qt_("List of Citations");
+		return qt_("Citations");
 	if (type == "label")
 		return qt_("Labels and References");
 	if (type == "branch")
-		return qt_("List of Branches");
+		return qt_("Branches");
 	if (type == "change")
-		return qt_("List of Changes");
+		return qt_("Changes");
 
 	FloatList const & floats = bp.documentClass().floats();
 	if (floats.typeExist(type))
