@@ -17,12 +17,15 @@
 
 #include "ColorCode.h"
 #include "InsetCode.h"
+#include "InsetLayout.h"
 #include "LayoutEnums.h"
 #include "OutputEnums.h"
 
 #include "support/strfwd.h"
 #include "support/types.h"
 
+#include <climits>
+
 
 namespace lyx {
 
@@ -124,7 +127,7 @@ public:
 	  * Intented purpose is to sanitize internal state with regard to current
 	  * Buffer. 
 	  **/
-	virtual void initView() {};
+	virtual void initView() {}
 	/// \return true if this inset is labeled.
 	virtual bool isLabeled() const { return false; }
 
@@ -313,7 +316,8 @@ public:
 	/// plain text output in ucs4 encoding
 	/// return the number of characters; in case of multiple lines of
 	/// output, add PLAINTEXT_NEWLINE to the number of chars in the last line
-	virtual int plaintext(odocstream &, OutputParams const &) const = 0;
+	virtual int plaintext(odocstringstream &, OutputParams const &,
+	                      size_t max_length = INT_MAX) const = 0;
 	/// docbook output
 	virtual int docbook(odocstream & os, OutputParams const &) const;
 	/// XHTML output
@@ -346,6 +350,8 @@ public:
 	virtual bool isActive() const { return nargs() > 0; }
 	/// can we click at the specified position ?
 	virtual bool clickable(int, int) const { return false; }
+	/// Move one cell backwards
+	virtual bool allowsCaptionVariation(std::string const &) const { return false; }
 
 	/// does this contain text that can be change track marked in DVI?
 	virtual bool canTrackChanges() const { return false; }
@@ -408,16 +414,18 @@ public:
 	
 	/// \return Context menu identifier. This function determines
 	/// whose Inset's menu should be shown for the given position.
-	virtual docstring contextMenu(BufferView const & bv, int x, int y) const;
+	virtual std::string contextMenu(BufferView const & bv, int x, int y) const;
 
 	/// \return Context menu identifier for this inset.
 	/// This default implementation returns an empty string.
-	virtual docstring contextMenuName() const;
+	virtual std::string contextMenuName() const;
 
 
 	virtual docstring layoutName() const;
 	///
 	virtual InsetLayout const & getLayout() const;
+	///
+	virtual bool isPassThru() const { return getLayout().isPassThru(); }
 	/// Is this inset's layout defined in the document's textclass?
 	bool undefined() const;
 	/// should this inset be handled like a normal character?
@@ -488,7 +496,7 @@ public:
 	/// Add an entry to the TocList
 	/// Pass a DocIterator that points at the paragraph containing
 	/// the inset
-	virtual void addToToc(DocIterator const &) const {}
+	virtual void addToToc(DocIterator const & /* di */, bool /* output_active */) const {}
 	/// Collect BibTeX information
 	virtual void collectBibKeys(InsetIterator const &) const {}
 	/// Update the counters of this inset and of its contents.
@@ -526,17 +534,18 @@ public:
 	/**
 	 * If this method returns true all explicitly set font attributes
 	 * are reset during editing operations.
-	 * For copy/paste the operations the language is never changed, since
+	 * For copy/paste operations the language is never changed, since
 	 * the language of a given text never changes if the text is
 	 * formatted differently, while other font attribues like size may
 	 * need to change if the text is copied from one environment to
 	 * another one.
-	 * If it returns false no font attribute is reset.
-	 * The default implementation returns the negation of inheritFont(),
+	 * If this method returns false no font attribute is reset.
+	 * The default implementation returns true if the resetFont layout
+	 * tag is set and otherwise the negation of inheritFont(),
 	 * since inherited inset font attributes do not need to be changed,
 	 * and non-inherited ones need to be set explicitly.
 	 */
-	virtual bool resetFontEdit() const { return !inheritFont(); }
+	virtual bool resetFontEdit() const;
 
 	/// set the change for the entire inset
 	virtual void setChange(Change const &) {}
