@@ -45,6 +45,8 @@ public:
 	docstring toolTip(BufferView const &, int, int) const
 		{ return tooltip_; }
 	///
+  docstring getTOCString() const;
+	///
 	bool hasSettings() const { return true; }
 	///
 	InsetCode lyxCode() const { return REF_CODE; }
@@ -53,7 +55,8 @@ public:
 	///
 	void latex(otexstream &, OutputParams const &) const;
 	///
-	int plaintext(odocstream &, OutputParams const &) const;
+	int plaintext(odocstringstream & ods, OutputParams const & op,
+	              size_t max_length = INT_MAX) const;
 	///
 	int docbook(odocstream &, OutputParams const &) const;
 	///
@@ -67,7 +70,7 @@ public:
 	///
 	void updateBuffer(ParIterator const & it, UpdateType);
 	///
-	void addToToc(DocIterator const &) const;
+	void addToToc(DocIterator const & di, bool output_active) const;
 	///
 	bool forceLTR() const { return true; }
 	//@}
