@@ -40,6 +40,7 @@
 
 #include "Bidi.h"
 #include "Buffer.h"
+#include "BufferParams.h"
 #include "BufferView.h"
 #include "CoordCache.h"
 #include "Cursor.h"
@@ -581,7 +582,7 @@ void InsetMathNest::doDispatch(Cursor & cur, FuncRequest & cmd)
 			size_t n = 0;
 			idocstringstream is(cmd.argument());
 			is >> n;
-			topaste = cap::selection(n);
+			topaste = cap::selection(n, buffer().params().documentClassPtr());
 		}
 		cur.niceInsert(topaste, parseflg, false);
 		cur.clearSelection(); // bug 393
@@ -729,7 +730,6 @@ void InsetMathNest::doDispatch(Cursor & cur, FuncRequest & cmd)
 	case LFUN_MOUSE_DOUBLE:
 	case LFUN_MOUSE_TRIPLE:
 	case LFUN_WORD_SELECT:
-	case LFUN_INSET_SELECT_ALL:
 		cur.pos() = 0;
 		cur.idx() = 0;
 		cur.resetAnchor();
@@ -1029,7 +1029,7 @@ void InsetMathNest::doDispatch(Cursor & cur, FuncRequest & cmd)
 	case LFUN_REGEXP_MODE: {
 		InsetMath * im = cur.inset().asInsetMath();
 		if (im) {
-			InsetMathHull * i = im->asHullInset();		
+			InsetMathHull * i = im->asHullInset();
 			if (i && i->getType() == hullRegexp) {
 				cur.message(_("Already in regular expression mode"));
 				break;
@@ -1092,7 +1092,7 @@ void InsetMathNest::doDispatch(Cursor & cur, FuncRequest & cmd)
 			&& name != "Bmatrix" && name != "vmatrix"
 			&& name != "Vmatrix" && name != "matrix")
 			name = from_ascii("matrix");
-		
+
 		cur.niceInsert(
 			MathAtom(new InsetMathAMSArray(buffer_, name, m, n)));
 		break;
@@ -1279,7 +1279,7 @@ void InsetMathNest::doDispatch(Cursor & cur, FuncRequest & cmd)
 		if (createInsetMath_fromDialogStr(cmd.argument(), ar)) {
 			cur.recordUndoSelection();
 			cur.insert(ar);
-			cur.forceBufferUpdate();			
+			cur.forceBufferUpdate();
 		} else
 			cur.undispatched();
 		break;
@@ -1455,7 +1455,7 @@ bool InsetMathNest::getStatus(Cursor & cur, FuncRequest const & cmd,
 	case LFUN_CAPTION_INSERT:
 		flag.setEnabled(false);
 		break;
-	
+
 	case LFUN_SPACE_INSERT: {
 		docstring const & name = cmd.argument();
 		if (name == "visible")
@@ -1635,9 +1635,8 @@ bool InsetMathNest::interpretChar(Cursor & cur, char_type const c)
 		}
 
 		// do not finish macro for known * commands
-		MathWordList const & mwl = mathedWordList();
 		bool star_macro = c == '*'
-			&& (mwl.find(name.substr(1) + "*") != mwl.end()
+			&& (in_word_set(name.substr(1) + '*')
 			    || cur.buffer()->getMacro(name.substr(1) + "*", cur, true));
 		if (isAlphaASCII(c) || star_macro) {
 			cur.activeMacro()->setName(name + docstring(1, c));
@@ -1730,7 +1729,7 @@ bool InsetMathNest::interpretChar(Cursor & cur, char_type const c)
 		cur.autocorrect() = false;
 		cur.message(_("Autocorrect Off ('!' to enter)"));
 		return true;
-	} 
+	}
 	if (lyxrc.autocorrection_math && c == '!' && !cur.autocorrect()) {
 		cur.autocorrect() = true;
 		cur.message(_("Autocorrect On (<space> to exit)"));
@@ -2183,8 +2182,11 @@ MathCompletionList::MathCompletionList(Cursor const & cur)
 	MathWordList::const_iterator it2;
 	//lyxerr << "Globals completion commands: ";
 	for (it2 = words.begin(); it2 != words.end(); ++it2) {
-		globals.push_back("\\" + (*it2).first);
-		//lyxerr << "\\" + (*it2).first << " ";
+		if (it2->second.inset != "macro") {
+			// macros are already read from MacroTable::globalMacros()
+			globals.push_back('\\' + it2->first);
+			//lyxerr << '\\' + it2->first << ' ';
+		}
 	}
 	//lyxerr << std::endl;
 	sort(globals.begin(), globals.end());
