@@ -14,6 +14,7 @@
 #include "MacroTable.h"
 #include "MathMacroTemplate.h"
 #include "MathMacroArgument.h"
+#include "MathParser.h"
 #include "MathStream.h"
 #include "MathSupport.h"
 #include "InsetMathNest.h"
@@ -41,13 +42,13 @@ namespace lyx {
 /////////////////////////////////////////////////////////////////////
 
 MacroData::MacroData(Buffer * buf)
-	: buffer_(buf), queried_(true), numargs_(0), optionals_(0), lockCount_(0),
-	  redefinition_(false), type_(MacroTypeNewcommand)
+	: buffer_(buf), queried_(true), numargs_(0), sym_(0), optionals_(0),
+	  lockCount_(0), redefinition_(false), type_(MacroTypeNewcommand)
 {}
 
 
 MacroData::MacroData(Buffer * buf, DocIterator const & pos)
-	: buffer_(buf), pos_(pos), queried_(false), numargs_(0),
+	: buffer_(buf), pos_(pos), queried_(false), numargs_(0), sym_(0),
 	  optionals_(0), lockCount_(0), redefinition_(false),
 	  type_(MacroTypeNewcommand)
 {
@@ -55,14 +56,14 @@ MacroData::MacroData(Buffer * buf, DocIterator const & pos)
 
 
 MacroData::MacroData(Buffer * buf, MathMacroTemplate const & macro)
-	: buffer_(buf), queried_(false), numargs_(0), optionals_(0), lockCount_(0),
-	  redefinition_(false), type_(MacroTypeNewcommand)
+	: buffer_(buf), queried_(false), numargs_(0), sym_(0), optionals_(0),
+	  lockCount_(0), redefinition_(false), type_(MacroTypeNewcommand)
 {
 	queryData(macro);
 }
 
 
-void MacroData::expand(vector<MathData> const & args, MathData & to) const
+bool MacroData::expand(vector<MathData> const & args, MathData & to) const
 {
 	updateData();
 
@@ -70,10 +71,10 @@ void MacroData::expand(vector<MathData> const & args, MathData & to) const
 	static InsetMathSqrt inset(0);
 	inset.setBuffer(const_cast<Buffer &>(*buffer_));
 
-	// FIXME UNICODE
-	asArray(display_.empty() ? definition_ : display_, inset.cell(0));
+	docstring const & definition(display_.empty() ? definition_ : display_);
+	asArray(definition, inset.cell(0));
 	//lyxerr << "MathData::expand: args: " << args << endl;
-	//lyxerr << "MathData::expand: ar: " << inset.cell(0) << endl;
+	//LYXERR0("MathData::expand: ar: " << inset.cell(0));
 	for (DocIterator it = doc_iterator_begin(buffer_, &inset); it; it.forwardChar()) {
 		if (!it.nextInset())
 			continue;
@@ -87,8 +88,12 @@ void MacroData::expand(vector<MathData> const & args, MathData & to) const
 			it.cell().insert(it.pos(), args[n - 1]);
 		}
 	}
-	//lyxerr << "MathData::expand: res: " << inset.cell(0) << endl;
+	//LYXERR0("MathData::expand: res: " << inset.cell(0));
 	to = inset.cell(0);
+	// If the result is equal to the definition then we either have a
+	// recursive loop, or the definition did not contain any macro in the
+	// first place.
+	return asString(to) != definition;
 }
 
 
@@ -106,6 +111,28 @@ vector<docstring> const & MacroData::defaults() const
 }
 
 
+string const MacroData::requires() const
+{
+	if (sym_)
+		return to_utf8(sym_->requires);
+	return string();
+}
+
+
+docstring const MacroData::xmlname() const
+{
+	if (sym_)
+		return sym_->xmlname;
+	return docstring();
+}
+
+
+char const * MacroData::MathMLtype() const
+{
+	return sym_ ? sym_->MathMLtype() : 0;
+}
+
+
 void MacroData::unlock() const
 {
 	--lockCount_;
@@ -206,12 +233,11 @@ MacroTable::insert(docstring const & name, MacroData const & data)
 
 
 MacroTable::iterator
-MacroTable::insert(Buffer * buf, docstring const & def, string const & requires)
+MacroTable::insert(Buffer * buf, docstring const & def)
 {
 	//lyxerr << "MacroTable::insert, def: " << to_utf8(def) << endl;
 	MathMacroTemplate mac(buf, def);
 	MacroData data(buf, mac);
-	data.requires() = requires;
 	return insert(mac.name(), data);
 }
 
