@@ -27,6 +27,7 @@ class Buffer;
 class MathData;
 class MathMacroTemplate;
 class Paragraph;
+class latexkeys;
 
 enum MacroType {
 	MacroTypeNewcommand,
@@ -52,15 +53,20 @@ public:
 	size_t numargs() const { updateData(); return numargs_; }
 	/// replace #1,#2,... by given MathAtom 0,1,.., _including_ the possible
 	/// optional argument
-	void expand(std::vector<MathData> const & from, MathData & to) const;
+	/// \return whether anything was expanded
+	bool expand(std::vector<MathData> const & from, MathData & to) const;
 	/// number of optional arguments
 	size_t optionals() const;
 	///
 	std::vector<docstring> const & defaults() const;
 	///
-	std::string const & requires() const { return requires_; }
+	std::string const requires() const;
 	///
-	std::string & requires() { return requires_; }
+	docstring const xmlname() const;
+	///
+	char const * MathMLtype() const;
+	///
+	void setSymbol(latexkeys const * sym) { sym_ = sym; }
 
 	/// lock while being drawn to avoid recursions
 	int lock() const { return ++lockCount_; }
@@ -85,7 +91,7 @@ public:
 		return definition_ == x.definition_
 			&& numargs_ == x.numargs_
 			&& display_ == x.display_
-			&& requires_ == x.requires_
+			&& sym_ == x.sym_
 			&& optionals_ == x.optionals_
 			&& defaults_ == x.defaults_;
 	}
@@ -117,7 +123,7 @@ private:
 	///
 	mutable docstring display_;
 	///
-	std::string requires_;
+	latexkeys const * sym_;
 	///
 	mutable size_t optionals_;
 	///
@@ -148,7 +154,7 @@ class MacroTable : public std::map<docstring, MacroData>
 {
 public:
 	/// Parse full "\\def..." or "\\newcommand..." or ...
-	iterator insert(Buffer * buf, docstring const & definition, std::string const &);
+	iterator insert(Buffer * buf, docstring const & definition);
 	/// Insert pre-digested macro definition
 	iterator insert(docstring const & name, MacroData const & data);
 	///
