@@ -34,12 +34,15 @@
 #include "frontends/Painter.h"
 
 #include "support/debug.h"
+#include "support/gettext.h"
 #include "support/lassert.h"
+#include "support/lstrings.h"
 #include "support/textutils.h"
 
 #include <ostream>
 #include <vector>
 
+using namespace lyx::support;
 using namespace std;
 
 namespace lyx {
@@ -49,11 +52,11 @@ namespace lyx {
 class ArgumentProxy : public InsetMath {
 public:
 	///
-	ArgumentProxy(MathMacro & mathMacro, size_t idx) 
+	ArgumentProxy(MathMacro & mathMacro, size_t idx)
 		: mathMacro_(mathMacro), idx_(idx) {}
 	///
-	ArgumentProxy(MathMacro & mathMacro, size_t idx, docstring const & def) 
-		: mathMacro_(mathMacro), idx_(idx) 
+	ArgumentProxy(MathMacro & mathMacro, size_t idx, docstring const & def)
+		: mathMacro_(mathMacro), idx_(idx)
 	{
 			asArray(def, def_);
 	}
@@ -78,13 +81,13 @@ public:
 	///
 	void draw(PainterInfo & pi, int x, int y) const {
 		if (mathMacro_.editMetrics(pi.base.bv)) {
-			// The only way a ArgumentProxy can appear is in a cell of the 
-			// MathMacro. Moreover the cells are only drawn in the DISPLAY_FOLDED 
-			// mode and then, if the macro is edited the monochrome 
+			// The only way a ArgumentProxy can appear is in a cell of the
+			// MathMacro. Moreover the cells are only drawn in the DISPLAY_FOLDED
+			// mode and then, if the macro is edited the monochrome
 			// mode is entered by the MathMacro before calling the cells' draw
 			// method. Then eventually this code is reached and the proxy leaves
-			// monochrome mode temporarely. Hence, if it is not in monochrome 
-			// here (and the assert triggers in pain.leaveMonochromeMode()) 
+			// monochrome mode temporarely. Hence, if it is not in monochrome
+			// here (and the assert triggers in pain.leaveMonochromeMode())
 			// it's a bug.
 			pi.pain.leaveMonochromeMode();
 			mathMacro_.cell(idx_).draw(pi, x, y);
@@ -99,17 +102,17 @@ public:
 	size_t idx() const { return idx_; }
 	///
 	int kerning(BufferView const * bv) const
-	{ 
+	{
 		if (mathMacro_.editMetrics(bv)
 		    || !mathMacro_.cell(idx_).empty())
-			return mathMacro_.cell(idx_).kerning(bv); 
+			return mathMacro_.cell(idx_).kerning(bv);
 		else
 			return def_.kerning(bv);
 	}
 
 private:
 	///
-	Inset * clone() const 
+	Inset * clone() const
 	{
 		return new ArgumentProxy(*this);
 	}
@@ -125,7 +128,8 @@ private:
 MathMacro::MathMacro(Buffer * buf, docstring const & name)
 	: InsetMathNest(buf, 0), name_(name), displayMode_(DISPLAY_INIT),
 		expanded_(buf), attachedArgsNum_(0), optionals_(0), nextFoldMode_(true),
-		macroBackup_(buf), macro_(0), needsUpdate_(false), appetite_(9)
+		macroBackup_(buf), macro_(0), needsUpdate_(false),
+		isUpdating_(false), appetite_(9)
 {}
 
 
@@ -211,15 +215,15 @@ void MathMacro::metrics(MetricsInfo & mi, Dimension & dim) const
 		dim.asc = max(bsdim.ascent(), dim.ascent());
 		dim.des = max(bsdim.descent(), dim.descent());
 		metricsMarkers(dim);
-	} else if (lyxrc.macro_edit_style == LyXRC::MACRO_EDIT_LIST 
+	} else if (lyxrc.macro_edit_style == LyXRC::MACRO_EDIT_LIST
 		   && editing_[mi.base.bv]) {
 		// Macro will be edited in a old-style list mode here:
 
-		LASSERT(macro_ != 0, /**/);
+		LBUFERR(macro_);
 		Dimension fontDim;
 		FontInfo labelFont = sane_font;
 		math_font_max_dim(labelFont, fontDim.asc, fontDim.des);
-		
+
 		// get dimension of components of list view
 		Dimension nameDim;
 		nameDim.wid = mathed_string_width(mi.base.font, from_ascii("Macro \\") + name() + ": ");
@@ -230,29 +234,29 @@ void MathMacro::metrics(MetricsInfo & mi, Dimension & dim) const
 		argDim.wid = mathed_string_width(labelFont, from_ascii("#9: "));
 		argDim.asc = fontDim.asc;
 		argDim.des = fontDim.des;
-		
+
 		Dimension defDim;
 		definition_.metrics(mi, defDim);
-		
+
 		// add them up
 		dim.wid = nameDim.wid + defDim.wid;
 		dim.asc = max(nameDim.asc, defDim.asc);
 		dim.des = max(nameDim.des, defDim.des);
-		
+
 		for (idx_type i = 0; i < nargs(); ++i) {
 			Dimension cdim;
 			cell(i).metrics(mi, cdim);
 			dim.des += max(argDim.height(), cdim.height()) + 1;
 			dim.wid = max(dim.wid, argDim.wid + cdim.wid);
 		}
-		
+
 		// make space for box and markers, 2 pixels
 		dim.asc += 1;
 		dim.des += 1;
 		dim.wid += 2;
 		metricsMarkers2(dim);
 	} else {
-		LASSERT(macro_ != 0, /**/);
+		LBUFERR(macro_);
 
 		// calculate metrics, hoping that all cells are seen
 		macro_->lock();
@@ -269,7 +273,7 @@ void MathMacro::metrics(MetricsInfo & mi, Dimension & dim) const
 		macro_->unlock();
 
 		// calculate dimension with label while editing
-		if (lyxrc.macro_edit_style == LyXRC::MACRO_EDIT_INLINE_BOX 
+		if (lyxrc.macro_edit_style == LyXRC::MACRO_EDIT_INLINE_BOX
 		    && editing_[mi.base.bv]) {
 			FontInfo font = mi.base.font;
 			augmentFont(font, from_ascii("lyxtex"));
@@ -284,7 +288,6 @@ void MathMacro::metrics(MetricsInfo & mi, Dimension & dim) const
 			dim.asc += 1 + namedim.height() + 1;
 			dim.des += 2;
 		}
-	 
 	}
 }
 
@@ -297,10 +300,10 @@ int MathMacro::kerning(BufferView const * bv) const {
 }
 
 
-void MathMacro::updateMacro(MacroContext const & mc) 
+void MathMacro::updateMacro(MacroContext const & mc)
 {
 	if (validName()) {
-		macro_ = mc.get(name());    
+		macro_ = mc.get(name());
 		if (macro_ && macroBackup_ != *macro_) {
 			macroBackup_ = *macro_;
 			needsUpdate_ = true;
@@ -311,40 +314,65 @@ void MathMacro::updateMacro(MacroContext const & mc)
 }
 
 
+class MathMacro::UpdateLocker
+{
+public:
+	explicit UpdateLocker(MathMacro & mm) : mac(mm)
+	{
+		mac.isUpdating_ = true;
+	}
+	~UpdateLocker() { mac.isUpdating_ = false; }
+private:
+	MathMacro & mac;
+};
+
+
 void MathMacro::updateRepresentation(Cursor * cur, MacroContext const & mc,
 		UpdateType utype)
 {
+	// block recursive calls (bug 8999)
+	if (isUpdating_)
+		return;
+
+	UpdateLocker locker(*this);
+
 	// known macro?
 	if (macro_ == 0)
 		return;
 
 	// update requires
 	requires_ = macro_->requires();
-	
+
 	if (!needsUpdate_
 		// non-normal mode? We are done!
 		|| (displayMode_ != DISPLAY_NORMAL))
 		return;
 
 	needsUpdate_ = false;
-	
+
 	// get default values of macro
 	vector<docstring> const & defaults = macro_->defaults();
-	
+
 	// create MathMacroArgumentValue objects pointing to the cells of the macro
 	vector<MathData> values(nargs());
 	for (size_t i = 0; i < nargs(); ++i) {
 		ArgumentProxy * proxy;
-		if (i < defaults.size()) 
+		if (i < defaults.size())
 			proxy = new ArgumentProxy(*this, i, defaults[i]);
 		else
 			proxy = new ArgumentProxy(*this, i);
 		values[i].insert(0, MathAtom(proxy));
 	}
 	// expanding macro with the values
-	macro_->expand(values, expanded_.cell(0));
-	if (utype == OutputUpdate && !expanded_.cell(0).empty())
-		expanded_.cell(0).updateMacros(cur, mc, utype);
+	// Only update the argument macros if anything was expanded, otherwise
+	// we would get an endless loop (bug 9140). UpdateLocker does not work
+	// in this case, since MacroData::expand() creates new MathMacro
+	// objects, so this would be a different recursion path than the one
+	// protected by UpdateLocker.
+	if (macro_->expand(values, expanded_.cell(0))) {
+		if (utype == OutputUpdate && !expanded_.cell(0).empty())
+			expanded_.cell(0).updateMacros(cur, mc, utype);
+	}
 	// get definition for list edit mode
 	docstring const & display = macro_->display();
 	asArray(display.empty() ? macro_->definition() : display, definition_);
@@ -359,7 +387,7 @@ void MathMacro::draw(PainterInfo & pi, int x, int y) const
 	int expx = x;
 	int expy = y;
 
-	if (displayMode_ == DISPLAY_INIT || displayMode_ == DISPLAY_INTERACTIVE_INIT) {		
+	if (displayMode_ == DISPLAY_INIT || displayMode_ == DISPLAY_INTERACTIVE_INIT) {
 		FontSetChanger dummy(pi.base, "lyxtex");
 		pi.pain.text(x, y, from_ascii("\\") + name(), pi.base.font);
 	} else if (displayMode_ == DISPLAY_UNFOLDED) {
@@ -371,17 +399,17 @@ void MathMacro::draw(PainterInfo & pi, int x, int y) const
 	} else if (lyxrc.macro_edit_style == LyXRC::MACRO_EDIT_LIST
 		   && editing_[pi.base.bv]) {
 		// Macro will be edited in a old-style list mode here:
-		
+
 		CoordCache const & coords = pi.base.bv->coordCache();
 		FontInfo const & labelFont = sane_font;
-		
+
 		// markers and box needs two pixels
 		x += 2;
-		
+
 		// get maximal font height
 		Dimension fontDim;
 		math_font_max_dim(pi.base.font, fontDim.asc, fontDim.des);
-		
+
 		// draw label
 		docstring label = from_ascii("Macro \\") + name() + from_ascii(": ");
 		pi.pain.text(x, y, label, labelFont);
@@ -391,38 +419,38 @@ void MathMacro::draw(PainterInfo & pi, int x, int y) const
 		definition_.draw(pi, x, y);
 		Dimension const & defDim = coords.getArrays().dim(&definition_);
 		y += max(fontDim.des, defDim.des);
-				
+
 		// draw parameters
 		docstring str = from_ascii("#9");
 		int strw1 = mathed_string_width(labelFont, from_ascii("#9"));
 		int strw2 = mathed_string_width(labelFont, from_ascii(": "));
-		
+
 		for (idx_type i = 0; i < nargs(); ++i) {
 			// position of label
 			Dimension const & cdim = coords.getArrays().dim(&cell(i));
 			x = expx + 2;
 			y += max(fontDim.asc, cdim.asc) + 1;
-			
+
 			// draw label
 			str[1] = '1' + i;
 			pi.pain.text(x, y, str, labelFont);
 			x += strw1;
 			pi.pain.text(x, y, from_ascii(":"), labelFont);
 			x += strw2;
-			
+
 			// draw paramter
 			cell(i).draw(pi, x, y);
-			
+
 			// next line
 			y += max(fontDim.des, cdim.des);
 		}
-		
-		pi.pain.rectangle(expx + 1, expy - dim.asc + 1, dim.wid - 3, 
+
+		pi.pain.rectangle(expx + 1, expy - dim.asc + 1, dim.wid - 3,
 				  dim.height() - 2, Color_mathmacroframe);
 		drawMarkers2(pi, expx, expy);
 	} else {
 		bool drawBox = lyxrc.macro_edit_style == LyXRC::MACRO_EDIT_INLINE_BOX;
-		
+
 		// warm up cells
 		for (size_t i = 0; i < nargs(); ++i)
 			cell(i).setXY(*pi.base.bv, x, y);
@@ -447,7 +475,7 @@ void MathMacro::draw(PainterInfo & pi, int x, int y) const
 			pi.pain.leaveMonochromeMode();
 
 			if (drawBox)
-				pi.pain.rectangle(x, y - dim.asc, dim.wid, 
+				pi.pain.rectangle(x, y - dim.asc, dim.wid,
 						  dim.height(), Color_mathmacroframe);
 		} else
 			expanded_.cell(0).draw(pi, expx, expy);
@@ -465,14 +493,14 @@ void MathMacro::draw(PainterInfo & pi, int x, int y) const
 void MathMacro::drawSelection(PainterInfo & pi, int x, int y) const
 {
 	// We may have 0 arguments, but InsetMathNest requires at least one.
-	if (cells_.size() > 0)
+	if (!cells_.empty())
 		InsetMathNest::drawSelection(pi, x, y);
 }
 
 
 void MathMacro::setDisplayMode(MathMacro::DisplayMode mode, int appetite)
 {
-	if (displayMode_ != mode) {		
+	if (displayMode_ != mode) {
 		// transfer name if changing from or to DISPLAY_UNFOLDED
 		if (mode == DISPLAY_UNFOLDED) {
 			cells_.resize(1);
@@ -485,7 +513,7 @@ void MathMacro::setDisplayMode(MathMacro::DisplayMode mode, int appetite)
 		displayMode_ = mode;
 		needsUpdate_ = true;
 	}
-	
+
 	// the interactive init mode is non-greedy by default
 	if (appetite == -1)
 		appetite_ = (mode == DISPLAY_INTERACTIVE_INIT) ? 0 : 9;
@@ -507,8 +535,7 @@ bool MathMacro::validName() const
 {
 	docstring n = name();
 
-	// empty name?
-	if (n.size() == 0)
+	if (n.empty())
 		return false;
 
 	// converting back and force doesn't swallow anything?
@@ -521,7 +548,7 @@ bool MathMacro::validName() const
 	for (size_t i = 0; i<n.size(); ++i) {
 		if (!(n[i] >= 'a' && n[i] <= 'z')
 		    && !(n[i] >= 'A' && n[i] <= 'Z')
-		    && n[i] != '*') 
+		    && n[i] != '*')
 			return false;
 	}
 
@@ -536,7 +563,7 @@ void MathMacro::validate(LaTeXFeatures & features) const
 
 	if (name() == "binom")
 		features.require("binom");
-	
+
 	// validate the cells and the definition
 	if (displayMode() == DISPLAY_NORMAL) {
 		definition_.validate(features);
@@ -557,7 +584,7 @@ Inset * MathMacro::editXY(Cursor & cur, int x, int y)
 	// We may have 0 arguments, but InsetMathNest requires at least one.
 	if (nargs() > 0) {
 		cur.screenUpdateFlags(Update::SinglePar);
-		return InsetMathNest::editXY(cur, x, y);		
+		return InsetMathNest::editXY(cur, x, y);
 	} else
 		return this;
 }
@@ -565,7 +592,7 @@ Inset * MathMacro::editXY(Cursor & cur, int x, int y)
 
 void MathMacro::removeArgument(Inset::pos_type pos) {
 	if (displayMode_ == DISPLAY_NORMAL) {
-		LASSERT(size_t(pos) < cells_.size(), /**/);
+		LASSERT(size_t(pos) < cells_.size(), return);
 		cells_.erase(cells_.begin() + pos);
 		if (size_t(pos) < attachedArgsNum_)
 			--attachedArgsNum_;
@@ -580,7 +607,7 @@ void MathMacro::removeArgument(Inset::pos_type pos) {
 
 void MathMacro::insertArgument(Inset::pos_type pos) {
 	if (displayMode_ == DISPLAY_NORMAL) {
-		LASSERT(size_t(pos) <= cells_.size(), /**/);
+		LASSERT(size_t(pos) <= cells_.size(), return);
 		cells_.insert(cells_.begin() + pos, MathData());
 		if (size_t(pos) < attachedArgsNum_)
 			++attachedArgsNum_;
@@ -594,7 +621,7 @@ void MathMacro::insertArgument(Inset::pos_type pos) {
 
 void MathMacro::detachArguments(vector<MathData> & args, bool strip)
 {
-	LASSERT(displayMode_ == DISPLAY_NORMAL, /**/);	
+	LASSERT(displayMode_ == DISPLAY_NORMAL, return);
 	args = cells_;
 
 	// strip off empty cells, but not more than arity-attachedArgsNum_
@@ -615,7 +642,7 @@ void MathMacro::detachArguments(vector<MathData> & args, bool strip)
 
 void MathMacro::attachArguments(vector<MathData> const & args, size_t arity, int optionals)
 {
-	LASSERT(displayMode_ == DISPLAY_NORMAL, /**/);
+	LASSERT(displayMode_ == DISPLAY_NORMAL, return);
 	cells_ = args;
 	attachedArgsNum_ = args.size();
 	cells_.resize(arity);
@@ -626,14 +653,14 @@ void MathMacro::attachArguments(vector<MathData> const & args, size_t arity, int
 }
 
 
-bool MathMacro::idxFirst(Cursor & cur) const 
+bool MathMacro::idxFirst(Cursor & cur) const
 {
 	cur.screenUpdateFlags(Update::SinglePar);
 	return InsetMathNest::idxFirst(cur);
 }
 
 
-bool MathMacro::idxLast(Cursor & cur) const 
+bool MathMacro::idxLast(Cursor & cur) const
 {
 	cur.screenUpdateFlags(Update::SinglePar);
 	return InsetMathNest::idxLast(cur);
@@ -648,7 +675,9 @@ bool MathMacro::notifyCursorLeaves(Cursor const & old, Cursor & cur)
 			// The macro name was changed
 			Cursor inset_cursor = old;
 			int macroSlice = inset_cursor.find(this);
-			LASSERT(macroSlice != -1, /**/);
+			// returning true means the cursor is "now" invalid,
+			// which it was.
+			LASSERT(macroSlice != -1, return true);
 			inset_cursor.cutOff(macroSlice);
 			inset_cursor.recordUndoInset();
 			inset_cursor.pop();
@@ -702,15 +731,16 @@ void MathMacro::write(WriteStream & os) const
 	}
 
 	// normal mode
-	LASSERT(macro_, /**/);
+	// we should be ok to continue even if this fails.
+	LATTEST(macro_);
 
 	// Always protect macros in a fragile environment
 	if (os.fragile())
 		os << "\\protect";
-	
+
 	os << "\\" << name();
 	bool first = true;
-	
+
 	// Optional arguments:
 	// First find last non-empty optional argument
 	idx_type emptyOptFrom = 0;
@@ -719,21 +749,21 @@ void MathMacro::write(WriteStream & os) const
 		if (!cell(i).empty())
 			emptyOptFrom = i + 1;
 	}
-	
+
 	// print out optionals
 	for (i=0; i < cells_.size() && i < emptyOptFrom; ++i) {
 		first = false;
 		os << "[" << cell(i) << "]";
 	}
-	
+
 	// skip the tailing empty optionals
 	i = optionals_;
-	
+
 	// Print remaining arguments
 	for (; i < cells_.size(); ++i) {
-		if (cell(i).size() == 1 
+		if (cell(i).size() == 1
 			&& cell(i)[0].nucleus()->asCharInset()
-			&& cell(i)[0].nucleus()->asCharInset()->getChar() < 0x80) {
+			&& isASCII(cell(i)[0].nucleus()->asCharInset()->getChar())) {
 			if (first)
 				os << " ";
 			os << cell(i);
@@ -756,6 +786,16 @@ void MathMacro::maple(MapleStream & os) const
 
 void MathMacro::mathmlize(MathStream & os) const
 {
+	LATTEST(macro_);
+	if (macro_) {
+		docstring const xmlname = macro_->xmlname();
+		if (!xmlname.empty()) {
+			char const * type = macro_->MathMLtype();
+			os << '<' << type << "> " << xmlname << " /<"
+			   << type << '>';
+			return;
+		}
+	}
 	MathData const & data = expanded_.cell(0);
 	if (data.empty()) {
 		// this means that we do not recognize the macro
@@ -767,6 +807,14 @@ void MathMacro::mathmlize(MathStream & os) const
 
 void MathMacro::htmlize(HtmlStream & os) const
 {
+	LATTEST(macro_);
+	if (macro_) {
+		docstring const xmlname = macro_->xmlname();
+		if (!xmlname.empty()) {
+			os << ' ' << xmlname << ' ';
+			return;
+		}
+	}
 	MathData const & data = expanded_.cell(0);
 	if (data.empty()) {
 		// this means that we do not recognize the macro
@@ -784,14 +832,13 @@ void MathMacro::octave(OctaveStream & os) const
 
 void MathMacro::infoize(odocstream & os) const
 {
-	os << "Macro: " << name();
+	os << bformat(_("Macro: %1$s"), name());
 }
 
 
 void MathMacro::infoize2(odocstream & os) const
 {
-	os << "Macro: " << name();
-
+	os << bformat(_("Macro: %1$s"), name());
 }
 
 
@@ -835,7 +882,7 @@ bool MathMacro::automaticPopupCompletion() const
 }
 
 
-CompletionList const * 
+CompletionList const *
 MathMacro::createCompletionList(Cursor const & cur) const
 {
 	if (displayMode() != DISPLAY_UNFOLDED)
@@ -852,7 +899,7 @@ docstring MathMacro::completionPrefix(Cursor const & cur) const
 
 	if (!completionSupported(cur))
 		return docstring();
-	
+
 	return "\\" + name();
 }
 
@@ -871,14 +918,14 @@ bool MathMacro::insertCompletion(Cursor & cur, docstring const & s,
 	asArray(newName, cell(0));
 	cur.bv().cursor().pos() = name().size();
 	cur.screenUpdateFlags(Update::SinglePar);
-	
+
 	// finish macro
 	if (finished) {
 		cur.bv().cursor().pop();
 		++cur.bv().cursor().pos();
 		cur.screenUpdateFlags(Update::SinglePar);
 	}
-	
+
 	return true;
 }
 
@@ -888,14 +935,14 @@ void MathMacro::completionPosAndDim(Cursor const & cur, int & x, int & y,
 {
 	if (displayMode() != DISPLAY_UNFOLDED)
 		InsetMathNest::completionPosAndDim(cur, x, y, dim);
-	
+
 	// get inset dimensions
 	dim = cur.bv().coordCache().insets().dim(this);
 	// FIXME: these 3 are no accurate, but should depend on the font.
 	// Now the popup jumps down if you enter a char with descent > 0.
 	dim.des += 3;
 	dim.asc += 3;
-	
+
 	// and position
 	Point xy
 	= cur.bv().coordCache().insets().xy(this);
